/* $Id: crires_model_anneal.c,v 1.13 2012-01-12 17:58:24 bristowp Exp $
 *
 * This file is part of the CRIRES Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: bristowp $
 * $Date: 2012-01-12 17:58:24 $
 * $Revision: 1.13 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                            Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>

#include "crires_model_anneal.h"
#include "crires_model_io.h"
#include "crires_model_kernel.h"
#include "crires_model_metric.h"
#include "crires_model_sa.h"

/*-----------------------------------------------------------------------------
                            Static
 -----------------------------------------------------------------------------*/

static int get_meas_coordinates(int, coord *, char *) ;
static int countlines(const char *) ;

/*----------------------------------------------------------------------------*/
/**
 * @defgroup crires_model_anneal    Anneal computation function
 */
/*----------------------------------------------------------------------------*/

/**@{*/

/*----------------------------------------------------------------------------*/
/**
  @brief    The actual anneal computation takes place here
  @param    cri_config_file
  @param    meas_coord_file
  @return   the new configuration table or NULL in error case
 */
/*----------------------------------------------------------------------------*/
cpl_table * crires_model_anneal_reduce(
        const char  *   cri_config_file,
        const char  *   meas_coord_file)
{
  cpl_table *   conf_tab ;
  int ii,morder,sizearraywavelengths, DS_size,adim;
  struct crires crires_model, *p_crires;
  ann_all_par all_par[300], *p_all_par;
  double abest[300];
  double amin[300];
  double amax[300];
  int aname[300];
  coord msp_coord[20000+5];

  DOUBLE flux_tot[100];
  int mode_tot[100];

  DOUBLE *p_wlarray;
  DOUBLE ** znse_ref;

  p_crires=&crires_model;
  p_all_par=&all_par[0];


  sizearraywavelengths=countlines(meas_coord_file);
  /* in multi-slit version sizearraywavelengths is the total number of wavelengths and slit positions*/
  printf("%d \n",sizearraywavelengths);
  //wavelength =  (DOUBLE *)cpl_malloc(sizearraywavelengths*sizeof(DOUBLE));

  if ((adim=crires_model_readfits(abest,
				  amin,
				  amax,
				  aname,
				  cri_config_file,
				  p_crires,
				  p_all_par))==-1) {
    cpl_msg_error(__func__, "Failed to read config FITS file" );
    return NULL;
  }

  /*znse_ref is the array holding Sellmeier coefs for
    ZnSe refractive index at each temp. Currently hard
    coded to take 6 temperatures in [2][*] to [7][*],
    can be extended. First two rows ([0][*] to [1][*])
    take the entries for the temperature bracketing the
    operation. Column [*][0] takes the temperature, the
    other columns, [*][1] to [*][6] take the Sellmeier
    coeffs */
  if ((znse_ref=crires_model_alloc2Darray(8,7))==NULL) {
    cpl_msg_error(__func__, "Cannot allocate 2D array");
    return NULL;
  }
  /*open and read refractive index data file */
  if (crires_model_ref_ind_read(znse_ref,p_crires->temper)==-1) {
    cpl_msg_error(__func__, "error setting ref index at T= %lf", p_crires->temper);
    return NULL ;
  }

  morder=50;
  /*We now need to update several matrices (that were set in the
    cri_init routine outside of the current loop) for this prism angle */
  if (crires_model_init(p_crires)!=0) {
    cpl_msg_error(__func__, "Failed to initialise model" );
    return NULL;
  }

  /* calculate size of the data structure*/
  /*for CRIRES DS_size is simply sizearraywavelengths,
    the CalcDataStructureSize function was only needed
    for the possibility of features occuring more than
    once due to order overlap*/

  //DS_size=CalcDataStructureSize(p_crires,p_Input,sizearraywavelengths,wavelength, znse_ref, temper, morder);
  DS_size = sizearraywavelengths;

  /* now allocate memory for the data structure*/
  p_wlarray=crires_model_alloc1Darray(DS_size);

  sizearraywavelengths=get_meas_coordinates(sizearraywavelengths, msp_coord,(char*)meas_coord_file);
  DS_size = sizearraywavelengths;

  for(ii=0;ii<sizearraywavelengths;ii++) {
    flux_tot[abs(msp_coord[ii].mode)]=0.0;
    mode_tot[abs(msp_coord[ii].mode)]=0;
  }
  for(ii=0;ii<sizearraywavelengths;ii++) {
    p_wlarray[ii]=msp_coord[ii].wave*1e-6;
    /* below is a quick hack to allow an offset in the
       co-ordinates eg 0.5 for pixel centres*/
    msp_coord[ii].x=msp_coord[ii].x-0.0;
    msp_coord[ii].y=msp_coord[ii].y-0.0;
    flux_tot[abs(msp_coord[ii].mode)]+=msp_coord[ii].flux;
    mode_tot[abs(msp_coord[ii].mode)]+=1;
  }
  for(ii=0;ii<sizearraywavelengths;ii++) {
    msp_coord[ii].flux=msp_coord[ii].flux*(float)(mode_tot[abs(msp_coord[ii].mode)])/flux_tot[abs(msp_coord[ii].mode)];
//    printf("%lf %d %d %lf\n",msp_coord[ii].flux,mode_tot[msp_coord[ii].mode],msp_coord[ii].mode,flux_tot[msp_coord[ii].mode]);
  }
  printf ("Before anneal: \n");
  for (ii=0;ii<adim;ii++) {
    printf("%d %s %lf \n", aname[ii], (p_all_par+aname[ii])->name, abest[ii]);
  }

  if ((conf_tab = crires_model_anneal_comp(p_all_par,
                           adim,
                           abest,
                           amin,
                           amax,
                           aname,
                           p_crires,
                           DS_size,
                           msp_coord,
                           p_wlarray,
                           znse_ref,
                           morder)) == NULL) {
      crires_model_free2Darray(znse_ref,8);
      cpl_free(p_wlarray);
      return NULL ;
  }
  printf ("After Anneal: \n");
  for (ii=0;ii<adim;ii++) {
    printf("%d %s %lf \n", aname[ii], (p_all_par+aname[ii])->name, abest[ii]);
  }
  if ((crires_model_free2Darray(znse_ref,8))!=0) {
    cpl_msg_error(__func__, "Cannot free 2D array");
    cpl_free(p_wlarray);
    return NULL;
  }
  cpl_free(p_wlarray);
  return conf_tab ;
}
/**@}*/

/*The number of measured co-ordinates must equal the number of wavelengths in meas_coord_file (and they should also correspond)*/
static int get_meas_coordinates(int sizearraywavelengths,
                          coord *meas_coord,
                          char *datname)
{
  int feat_cnt=0;
  FILE * measfile=fopen(datname,"r");
  /*  FILE * matchfile=fopen("match.dat","r");*/
  if(measfile ==NULL ){
    cpl_msg_error(__func__, "Cannot open the measured coords file");
    return -1;
  }
  while(fscanf(measfile, "%d %lf %lf %d %lf %d %lf %d %d %d %lf\n",
               &meas_coord[feat_cnt].counter,
               &meas_coord[feat_cnt].y,
               &meas_coord[feat_cnt].x,
               &meas_coord[feat_cnt].chip,
               &meas_coord[feat_cnt].flux,
               &meas_coord[feat_cnt].slit_pos,
               &meas_coord[feat_cnt].wave,
               &meas_coord[feat_cnt].mode,
               &meas_coord[feat_cnt].penc,
               &meas_coord[feat_cnt].genc,
	       &meas_coord[feat_cnt].slitw) != EOF ){
    if (meas_coord[feat_cnt].x>10.0 && meas_coord[feat_cnt].x<1014.0) {
      if (meas_coord[feat_cnt].counter<6000 && meas_coord[feat_cnt].counter<6000 && meas_coord[feat_cnt].chip!=9 && meas_coord[feat_cnt].slit_pos!=9 && ((meas_coord[feat_cnt].wave>900.0 && meas_coord[feat_cnt].wave<2700.0))) {// || (meas_coord[feat_cnt].wave>1300.0 && meas_coord[feat_cnt].wave<1515.0) || meas_coord[feat_cnt].wave>1700.0)) {
	feat_cnt+=1;
      }
    }
  }

  if (feat_cnt != sizearraywavelengths) {
    cpl_msg_error(__func__, "Number of spectral features is not equal to the number of wavelengths and slit posn's supplied! \n %d != %d \n",feat_cnt, sizearraywavelengths);
    //return -1;
  }
  return feat_cnt;
}

/*count lines of the calibration files in order to dimension the array's size.*/ 
static int countlines (const char * filename)
{
  /* count  lines  ignore Comments (lines start with a %)*/
  char oneline[200];
  char *c;
  char firstchar[]="";
  char comment[]="%";
  FILE * myfile;
  int linecounter=0;
  if((myfile=fopen((char*)filename,"r"))==NULL){
    cpl_msg_error(__func__, "Cannot open file %s for reading.\n",filename);
    return -1;
  }

  do {
    c = fgets(oneline, 200, myfile);  /* get one line from the file */
    if (c != NULL)
      {
	strncpy(firstchar,oneline,1);
	if(strncmp(firstchar,comment,1))
	  /*if is not a comment increment linecounter*/
	  linecounter++;
      }
    
  }
  while (c != NULL);               /* repeat until NULL          */
  fclose(myfile);
  return linecounter;
}





