/* $Id: crires_model_fix.c,v 1.22 2011-03-22 09:17:12 yjung Exp $
 *
 * This file is part of the CRIRES Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: yjung $
 * $Date: 2011-03-22 09:17:12 $
 * $Revision: 1.22 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include <locale.h>
#include "crires_recipe.h"

#include "crires_model_refining.h"
#include "crires_model_kernel.h"

/*-----------------------------------------------------------------------------
                                Define
 -----------------------------------------------------------------------------*/

#define RECIPE_STRING "crires_model_fix"

/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static int crires_model_fix_parse(const char *, cpl_vector **, cpl_vector **, 
        int **, cpl_vector **) ;
static int crires_model_fix_save(cpl_table *, const cpl_parameterlist *, 
        cpl_frameset *) ;

static char crires_model_fix_description[] =
"crires_model_fix -- Model Fixing recipe\n"
"The files listed in the Set Of Frames (sof-file) must be tagged:\n"
"raw-file.fits "CRIRES_MODEL_FIX_RAW" or\n"
"WLS-file.txt "CRIRES_CALPRO_MODEL_FIX_TAB" like:\n"
"  The TXT file is something like:\n"
"  #   pos_x   pos_y   chip    wls\n"
"      500     100     3       1025.0\n"
"      600     100     3       1035.0\n"
"      700     100     3       1045.0\n"
"      900     100     3       1065.0\n"
"config-file.fits "CRIRES_CALPRO_MODEL_CONFIG" or\n"
"config-file.fits "CRIRES_CALPRO_MODEL_REFINE_CONF".\n" ;

CRIRES_RECIPE_DEFINE(crires_model_fix,
        0,
        "Model Fixing recipe",
        crires_model_fix_description) ;

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    parlist     the parameters list
  @param    frameset   the frames list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_model_fix(
        cpl_frameset            *   frameset,
        const cpl_parameterlist *   parlist)
{
    cpl_frameset    *   rawframes ;
    const char      *   config ;
    const char      *   user_wls ;
    cpl_frame       *   ref_frame ;
    const char      *   ref_fname ;
    cpl_vector      *   pos_x ;
    cpl_vector      *   pos_y ;
    int             *   chip ;
    cpl_vector      *   wls ;
    cpl_table       *   new_conf ;

    /* Needed for sscanf() */
    setlocale(LC_NUMERIC, "C");

    /* The Model is switched off */
    if (crires_model_off()) {
        return 0 ;
    }

    /* Initialise */
    rawframes = NULL ;
    
    /* Retrieve input parameters */
 
    /* Identify the RAW and CALIB frames in the input frameset */
    if (crires_dfs_set_groups(frameset, "crires_model_fix")) {
        cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
        return -1 ;
    }

    /* Retrieve calibration data */
    config = crires_extract_filename(frameset, CRIRES_CALPRO_MODEL_CONFIG) ;
    if (config == NULL) {
        config = crires_extract_filename(frameset, 
                CRIRES_CALPRO_MODEL_REFINE_CONF) ;
    }
    if (config == NULL) {
        cpl_msg_error(__func__, "Model configuration file is missing") ;
        return -1 ;
    }
    user_wls = crires_extract_filename(frameset, CRIRES_CALPRO_MODEL_FIX_TAB) ;
    if (user_wls == NULL) {
        cpl_msg_error(__func__, "User specified wavelengths are missing") ;
        return -1 ;
    }

    /* Retrieve raw frames */
    if ((rawframes = crires_extract_frameset(frameset, 
                    CRIRES_MODEL_FIX_RAW)) == NULL) {
        cpl_msg_error(__func__, "No raw frame in input") ;
        return -1 ;
    }
    ref_frame = cpl_frameset_get_position(rawframes, 0) ;
    ref_fname = cpl_frame_get_filename(ref_frame) ;

    /* Check the model config file validity vs. the current data  */
    if (crires_model_config_check(config, ref_fname) != 0) {
        cpl_msg_error(__func__, 
                "The model configuration file version is wrong") ;
        cpl_frameset_delete(rawframes) ;
        return -1 ;
    }

    /* Read the input TXT file */
    cpl_msg_info(__func__, "Parse the passed TXT file") ;
    if (crires_model_fix_parse(user_wls, &pos_x, &pos_y, &chip, &wls)) {
        cpl_msg_error(__func__, "Cannot parse the input txt file") ;
        cpl_frameset_delete(rawframes) ;
        return -1 ;
    }

    /* Compute the new configuration file */
    cpl_msg_info(__func__, "Compute the new configuration file") ;
    cpl_msg_indent_more() ;
    if ((new_conf = crires_model_refining_fix(pos_x, pos_y, chip, wls,
                    ref_fname, config)) == NULL) {
        cpl_msg_error(__func__, "Cannot compute the new configuration file");
        cpl_frameset_delete(rawframes) ;
        cpl_vector_delete(pos_x) ;
        cpl_vector_delete(pos_y) ;
        cpl_free(chip) ;
        cpl_vector_delete(wls) ;
        cpl_msg_indent_less() ;
        return -1; 
    }
    cpl_msg_indent_less() ;

    /* Clean */
    cpl_vector_delete(pos_x) ;
    cpl_vector_delete(pos_y) ;
    cpl_free(chip) ;
    cpl_vector_delete(wls) ;
    cpl_frameset_delete(rawframes) ;
    
    /* Save the new configuration file */
    cpl_msg_info(__func__, "Save the new configuration file") ;
    if (crires_model_fix_save(new_conf, parlist, frameset) == -1) {
        cpl_msg_error(__func__, "Cannot save the product");
        cpl_table_delete(new_conf) ;
        return -1 ;
    }
    cpl_table_delete(new_conf) ;
   
    /* Return */
    if (cpl_error_get_code()) return -1 ;
    else return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Save the product of the recipe
  @param    out_table   the table 
  @param    parlist     the input list of parameters
  @param    set         the input frame set
  @return   0 if everything is ok, -1 otherwise
 */
/*----------------------------------------------------------------------------*/
static int crires_model_fix_save(
        cpl_table               *   out_table,
        const cpl_parameterlist *   parlist,
        cpl_frameset            *   set)
{
    cpl_propertylist    *   plist ;

    plist = cpl_propertylist_new();
    cpl_propertylist_append_string(plist, "INSTRUME", "CRIRES") ;
    cpl_propertylist_append_string(plist, CPL_DFS_PRO_CATG,
            CRIRES_CALPRO_MODEL_CONFIG) ;
    cpl_propertylist_append_string(plist, CPL_DFS_PRO_TYPE,
            CRIRES_PROTYPE_MOD_CONF) ;

    /* Use standard saving function */
    if (cpl_dfs_save_table(set, NULL, parlist, set, NULL, out_table,
                NULL, "crires_model_fix", plist, NULL, 
                PACKAGE "/" PACKAGE_VERSION,
                "crires_model_fix.fits") != CPL_ERROR_NONE) {
        cpl_msg_error(__func__, "Cannot save the table") ;
        return -1 ;
    }
    cpl_propertylist_delete(plist) ;

    /* Return */
    return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Parse the input txt file
  @param    txt_file    The file name
  @param    pos_x
  @param    pos_y
  @param    chip
  @param    wls
  @return   0 if everything is ok, -1 otherwise

  The TXT file is something like:

  #   pos_x   pos_y   chip    wls
      500     100     3       1025.0
      600     100     3       1035.0
      700     100     3       1045.0
      900     100     3       1065.0
 */
/*----------------------------------------------------------------------------*/
static int crires_model_fix_parse(
        const char  *   txt_file,
        cpl_vector  **  pos_x,
        cpl_vector  **  pos_y,
        int         **  chip,
        cpl_vector  **  wls)
{
    FILE        *   in;
    int             size ;
    char            line[1024];
    double          x, y, wl;
    int             ch ;

    /* Test entries */
    if (txt_file == NULL) return -1 ;
    if (pos_x == NULL) return -1 ;
    if (pos_y == NULL) return -1 ;
    if (chip == NULL) return -1 ;
    if (wls == NULL) return -1 ;

    /* Open the file */
    in = fopen(txt_file, "r");
    if (in == NULL) return -1 ;

    /* Get the size */
    size = 0 ;
    while (fgets(line, 1024, in) != NULL) {
        if (line[0] != '#' && sscanf(line, "%lg %lg %d %lg", &x, &y,
                    &ch, &wl) == 4) {
            size ++ ;
        }
    }

    /* Create and fill the vectors */
    *pos_x = cpl_vector_new(size);
    *pos_y = cpl_vector_new(size);
    *chip = cpl_malloc(size*sizeof(int)) ;
    *wls = cpl_vector_new(size);

    size = 0 ;
    rewind(in) ;
    while (fgets(line, 1024, in) != NULL) {
        if (line[0] != '#' && sscanf(line, "%lg %lg %d %lg", &x, &y,
                    &ch, &wl) == 4) {
            cpl_vector_set(*pos_x, size, x);
            cpl_vector_set(*pos_y, size, y);
            (*chip)[size] = ch ;
            cpl_vector_set(*wls, size, wl);
            size ++ ;
        }
    }
    fclose(in);

    return 0 ;
}
