/* $Id: crires_model_wlmap.c,v 1.35 2011-03-22 09:17:12 yjung Exp $
 *
 * This file is part of the CRIRES Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: yjung $
 * $Date: 2011-03-22 09:17:12 $
 * $Revision: 1.35 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include "crires_recipe.h"

#include "crires_model_kernel.h"

/*-----------------------------------------------------------------------------
                                Define
 -----------------------------------------------------------------------------*/

#define RECIPE_STRING "crires_model_wlmap"

/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static int crires_model_wlmap_save(const cpl_imagelist *,
        const cpl_parameterlist *, cpl_frameset *) ;

static char crires_model_wlmap_description[] =
"crires_model_wlmap -- Wavelength map creation using the model\n"
"The files listed in the Set Of Frames (sof-file) must be tagged:\n"
"raw-file.fits "CRIRES_MODEL_WLMAP_RAW" or\n"
"config-file.fits "CRIRES_CALPRO_MODEL_CONFIG" or\n"
"config-file.fits "CRIRES_CALPRO_MODEL_REFINE_CONF".\n" ;

CRIRES_RECIPE_DEFINE(crires_model_wlmap,
        CRIRES_PARAM_ORDER,
        "Model wavelength map recipe",
        crires_model_wlmap_description) ;

/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static struct {
    /* Inputs */
    int             order ;  
    /* Outputs */
    crires_illum_period period ;
} crires_model_wlmap_config ;

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    parlist     the parameters list
  @param    frameset   the frames list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_model_wlmap(
        cpl_frameset            *   frameset,
        const cpl_parameterlist *   parlist)
{
    cpl_frameset    *   rawframes ;
    const char      *   config ;
    cpl_frame       *   ref_frame ;
    cpl_imagelist   *   wlmap ;
    //cpl_vector   *   profile ; 

    /* The Model is switched off */
    if (crires_model_off()) {
        return 0 ;
    }

    /* Initialise */
    rawframes = NULL ;
    
    /* Retrieve input parameters */
    crires_model_wlmap_config.order = crires_parameterlist_get_int(parlist,
            RECIPE_STRING, CRIRES_PARAM_ORDER) ;
 
    /* Identify the RAW and CALIB frames in the input frameset */
    if (crires_dfs_set_groups(frameset, "crires_model_wlmap")) {
        cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
        return -1 ;
    }

    /* Retrieve calibration data */
    config = crires_extract_filename(frameset, CRIRES_CALPRO_MODEL_CONFIG) ;
    if (config == NULL) {
        config = crires_extract_filename(frameset,
                CRIRES_CALPRO_MODEL_REFINE_CONF) ;
    }
    if (config == NULL) {
        cpl_msg_error(__func__, "No config file in input") ;
        return -1 ;
    }

    /* Retrieve raw frames */
    if ((rawframes = crires_extract_frameset(frameset, 
                    CRIRES_MODEL_WLMAP_RAW)) == NULL) {
        cpl_msg_error(__func__, "No raw frame in input") ;
        return -1 ;
    }
    ref_frame = cpl_frameset_get_position(rawframes, 0) ;

    /* Check the model config file validity vs. the current data  */
    if (crires_model_config_check(config, 
                cpl_frame_get_filename(ref_frame)) != 0) {
        cpl_msg_error(__func__, 
                "The model configuration file version is wrong") ;
        cpl_frameset_delete(rawframes) ;
        return -1 ;
    }

    /* Get the detector illumination period */
    crires_model_wlmap_config.period =
        crires_get_detector_illum_period(cpl_frame_get_filename(ref_frame)) ;
    if (crires_model_wlmap_config.period == CRIRES_ILLUM_UNKNOWN) {
        cpl_msg_error(__func__,
                "Cannot determine the detector illumination period") ;
        cpl_frameset_delete(rawframes) ;
        return -1 ;
    } else {
        crires_display_detector_illum(crires_model_wlmap_config.period) ;
    }

    /*    if ((profile=crires_model_profile(cpl_frame_get_filename(ref_frame),
				      config, -1,
				      1000.0)) == NULL) {
      cpl_msg_error(__func__, "Cannot compute the profile") ;
      cpl_frameset_delete(rawframes) ;
      return -1 ;
    }
    if ((line_list=crires_model_predict(cpl_frame_get_filename(ref_frame),
					"ThAr_use.fits",
					config,-1)) == NULL) {
      cpl_msg_error(__func__, "cannot predict the line positions") ;
      cpl_frameset_delete(rawframes) ;
      return -1 ;
    }
    int ii=0;
    while (line_list[0][ii]!=0.0) {
      printf("here %lf %lf %lf %lf %lf %lf \n",line_list[0][ii],line_list[1][ii],line_list[2][ii],line_list[3][ii],line_list[4][ii],line_list[5][ii]);
      ii+=1;
    }
    crires_model_free2Darray(line_list,6);
    cpl_vector_delete(profile);
    */

    /* Compute the wavelength map */
    if ((wlmap = crires_model_wavpix(cpl_frame_get_filename(ref_frame),
                    config, crires_model_wlmap_config.order)) == NULL) {
        cpl_msg_error(__func__, "Cannot compute the WL map") ;
        cpl_frameset_delete(rawframes) ;
        return -1 ;
    }
    cpl_frameset_delete(rawframes) ;

    /* Save the product */
    if (crires_model_wlmap_save(wlmap, parlist, frameset) == -1) {
        cpl_msg_error(__func__, "Cannot save products") ;
        cpl_imagelist_delete(wlmap) ;
        return -1 ;
    }
    cpl_imagelist_delete(wlmap) ;
    
    /* Return */
    if (cpl_error_get_code()) return -1 ;
    else return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Save the products
  @param    wlmap       the wavelength map
  @param    parlist     the parameters list
  @param    set         the frames set
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_model_wlmap_save(
        const cpl_imagelist     *   wlmap,
        const cpl_parameterlist *   parlist,
        cpl_frameset            *   set)
{
    const char          *   recipe_name = "crires_model_wlmap" ;

    /* Write the flat image */
    crires_image_save(set,
            parlist,
            set, 
            wlmap, 
            recipe_name,
            CRIRES_WL_MAP_MODEL_IMA, 
            CRIRES_PROTYPE_WL_MAP,
            crires_model_wlmap_config.period,
            NULL,
            NULL,
            PACKAGE "/" PACKAGE_VERSION,
            "crires_model_wlmap.fits") ;

    return  0;
}
