/* $Id: crires_util_conversion.c,v 1.37 2012-09-19 14:10:27 yjung Exp $
 *
 * This file is part of the crires Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: yjung $
 * $Date: 2012-09-19 14:10:27 $
 * $Revision: 1.37 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include "crires_recipe.h"

#include "crires_photom.h"
#include "irplib_utils.h"

/*-----------------------------------------------------------------------------
                                Define
 -----------------------------------------------------------------------------*/

#define RECIPE_STRING "crires_util_conversion"

/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static int crires_util_conversion_save(const cpl_table **, 
        const cpl_parameterlist *, cpl_frameset *) ;

static char crires_util_conversion_description[] = 
"This recipe accepts 2 parameters:\n"
"First parameter:   the table of the extracted spectrum in wavelength.\n"
"                   (PRO TYPE = "CRIRES_PROTYPE_SPEC_WL")\n"
"Second parameter:  the table with the standard star photospheric flux.\n"
"                   (PRO TYPE = "CRIRES_PROTYPE_PHO_FLUX")\n"
"\n"
"This recipe produces 1 file:\n"
"First product:     the table with the conversion signal.\n"
"                   (PRO TYPE = "CRIRES_PROTYPE_CONVERS")\n" ;

CRIRES_RECIPE_DEFINE(crires_util_conversion,
        CRIRES_PARAM_DISPLAY,
        "Compute the conversion factor",
        crires_util_conversion_description) ;

/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static struct {
    /* Inputs */
    int             display ;
    /* Outputs */
    int             win_mode ;
    double          conv_med[CRIRES_NB_DETECTORS] ;
    double          thro_med[CRIRES_NB_DETECTORS] ;
} crires_util_conversion_config ;

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    frameset   the frames list
  @param    parlist     the parameters list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_util_conversion(
        cpl_frameset            *   frameset,
        const cpl_parameterlist *   parlist)
{
    cpl_frame           *   fr ;
    const char          *   sval ;
    cpl_propertylist    *   plist ;
    cpl_table           *   ext_tab[CRIRES_NB_DETECTORS] ;
    cpl_table           *   std_flux ;
    cpl_bivector        *   std_biv ;
    int                     i, j ;

    /* Initialise */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        crires_util_conversion_config.conv_med[i] = -1.0 ;
        crires_util_conversion_config.thro_med[i] = -1.0 ;
        ext_tab[i] = NULL ;
    }
 
    /* Retrieve input parameters */
    crires_util_conversion_config.display = crires_parameterlist_get_int(
            parlist, RECIPE_STRING, CRIRES_PARAM_DISPLAY) ;
    
    /* Identify the RAW and CALIB frames in the input frameset */
    if (crires_dfs_set_groups(frameset, NULL)) {
        cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
        return -1 ;
    }

    /* The second frame must be the std star flux */
    cpl_msg_info(__func__, "Second frame validity check") ;
    cpl_msg_indent_more() ;
    fr = cpl_frameset_get_position(frameset, 1);
    if ((std_flux = crires_load_table_check(cpl_frame_get_filename(fr), 1,
                    CRIRES_PROTYPE_PHO_FLUX, -1, -1, 0)) == NULL) {
        cpl_msg_error(__func__, "Second frame PRO TYPE must be %s",
                CRIRES_PROTYPE_PHO_FLUX) ;
        cpl_msg_indent_less() ;
        return -1 ;
    }
    cpl_msg_indent_less() ;
 
    /* Get the requested star */
    cpl_msg_info(__func__, "Standard star photospheric flux retrieval") ;
    cpl_msg_indent_more() ;
    if ((std_biv = crires_photom_conv_get_star(std_flux, 
        cpl_frame_get_filename(cpl_frameset_get_position(frameset, 0))))==NULL){
        cpl_msg_error(__func__, "Cannot find the star") ;
        cpl_msg_indent_less() ;
        cpl_table_delete(std_flux) ;
        return -1 ;
    }
    cpl_table_delete(std_flux) ;
    cpl_msg_indent_less() ;
   
    /* Windowing mode ? */
    fr = cpl_frameset_get_position(frameset, 0);
    if ((plist=cpl_propertylist_load(cpl_frame_get_filename(fr), 0)) == NULL)
        return -1 ;
    sval = crires_pfits_get_ncorrs(plist) ;
    if (!strcmp(sval, "FowlerNsampGRstWin")) {
        crires_util_conversion_config.win_mode = 1 ;
    } else {
        crires_util_conversion_config.win_mode = 0 ;
    }
    cpl_propertylist_delete(plist) ;

    /* Loop on the CRIRES_NB_DETECTORS detectors */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        /* Skip some detectors in windowing mode */
        if ((i==0 || i==CRIRES_NB_DETECTORS-1) &&
                (crires_util_conversion_config.win_mode == 1)) {
            continue ;
        }

        cpl_msg_info(__func__, "Conversion factor for chip %d", i+1) ;
        
        /* The first frame must be the extracted spectrum in wavelength */
        cpl_msg_info(__func__, "Load the extracted spectrum") ;
        cpl_msg_indent_more() ;
        fr = cpl_frameset_get_position(frameset, 0);
        if ((ext_tab[i] = crires_load_table_check(cpl_frame_get_filename(fr),
                        i+1, CRIRES_PROTYPE_SPEC_WL, -1, -1, 0)) == NULL) {
            cpl_msg_indent_less() ;
            continue ;
        }
        cpl_msg_indent_less() ;
           
        /* Compute */
        cpl_msg_info(__func__, "Conversion factor computation") ;
        cpl_msg_indent_more() ;
        if (crires_photom_conv_engine(ext_tab[i], std_biv, i+1,
                    crires_util_conversion_config.display==i+1) == -1) {
            cpl_msg_error(__func__, "Cannot compute conversion factor") ;
            cpl_msg_indent_less() ;
            cpl_bivector_delete(std_biv) ;
            for (j=0 ; j<i ; j++) {
                if (ext_tab[j] != NULL) cpl_table_delete(ext_tab[j]) ;
            }
            return -1 ;
        }
        cpl_msg_indent_less() ;
        
        /* Compute the Median */
        crires_util_conversion_config.conv_med[i] =
            cpl_table_get_column_median(ext_tab[i], CRIRES_COL_CONVERSION_RECT);
        crires_util_conversion_config.thro_med[i] =
            cpl_table_get_column_median(ext_tab[i], CRIRES_COL_THROUGHPUT) ;
    }
    cpl_bivector_delete(std_biv) ;
    
    /* Reconstruct chips in windowing mode using detector 2 */
    if (crires_util_conversion_config.win_mode == 1) {
        if (ext_tab[1] != NULL) {
            ext_tab[0] = cpl_table_duplicate(ext_tab[1]) ;
            cpl_table_set_size(ext_tab[0], 0) ;
            ext_tab[CRIRES_NB_DETECTORS-1] = cpl_table_duplicate(ext_tab[0]) ;
        }
    }

    /* Save the product */
    cpl_msg_info(__func__, "Save the product") ;
    cpl_msg_indent_more() ;
    if (crires_util_conversion_save((const cpl_table **)ext_tab, parlist, 
                frameset)) {
        cpl_msg_error(__func__, "Cannot save the product") ;
        for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
            if (ext_tab[i] != NULL) cpl_table_delete(ext_tab[i]) ;
        }
        cpl_msg_indent_less() ;
        return -1 ;
    }
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        if (ext_tab[i] != NULL) cpl_table_delete(ext_tab[i]) ;
    }
    cpl_msg_indent_less() ;

    /* Return */
    if (cpl_error_get_code()) return -1 ;
    else return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Save the products
  @param    out         the tables to save
  @param    parlist     the parameters list
  @param    set         the frames set
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_util_conversion_save(
        const cpl_table         **  out,
        const cpl_parameterlist *   parlist,
        cpl_frameset            *   set)
{
    cpl_propertylist    **  qclists ;
    const cpl_frame     *   ref_frame ;
    cpl_propertylist    *   inputlist ;
    const char          *   recipe_name = "crires_util_conversion" ;
    int                     i ;

    /* Get the reference frame */
    ref_frame = irplib_frameset_get_first_from_group(set, CPL_FRAME_GROUP_RAW) ;

    /* Create the QC lists */
    qclists = cpl_malloc(CRIRES_NB_DETECTORS * sizeof(cpl_propertylist*)) ;
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        qclists[i] = cpl_propertylist_new() ;
        cpl_propertylist_append_double(qclists[i], "ESO QC CONVMED",
                crires_util_conversion_config.conv_med[i]) ;
        cpl_propertylist_append_double(qclists[i], "ESO QC THROMED",
                crires_util_conversion_config.thro_med[i]) ;

        /* Propagate some keywords from input raw frame extensions */
        inputlist = cpl_propertylist_load_regexp(
                cpl_frame_get_filename(ref_frame), i+1,
                CRIRES_HEADER_EXT_FORWARD, 0) ;
        cpl_propertylist_copy_property_regexp(qclists[i], inputlist, 
                CRIRES_HEADER_EXT_FORWARD, 0) ;
        cpl_propertylist_delete(inputlist) ;
    }
    
    /* Write the table */
    crires_table_save(set,
            parlist,
            set,
            out,
            recipe_name,
            CRIRES_EXTRACT_CONV_TAB,
            CRIRES_PROTYPE_CONVERS,
            NULL,
            (const cpl_propertylist **)qclists,
            PACKAGE "/" PACKAGE_VERSION,
            "crires_util_conversion.fits") ;

    /* Free and return */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        cpl_propertylist_delete(qclists[i]) ;
    }
    cpl_free(qclists) ;
    return  0;
}

