/* $Id: crires_util_plot.c,v 1.33 2012-09-19 14:10:27 yjung Exp $
 *
 * This file is part of the crires Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: yjung $
 * $Date: 2012-09-19 14:10:27 $
 * $Revision: 1.33 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include "crires_recipe.h"

#include "irplib_wlxcorr.h"

/*-----------------------------------------------------------------------------
                                Define
 -----------------------------------------------------------------------------*/

#define RECIPE_STRING "crires_util_plot"

/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static int crires_util_plot_diff(const cpl_table *,const char *, const char *) ;
static int crires_util_plot_spec_pix(cpl_table *, cpl_table *, int) ;
static int crires_util_plot_spec_pix_one(cpl_table *, cpl_table *, int, 
        const char *, const char *, const char *) ;
static int crires_util_plot_spec_wl(cpl_table *, cpl_table *, 
        const char *, int) ;
static int crires_util_plot_convers(cpl_table *, cpl_table *, 
        const char *, int) ;
static int crires_util_plot_sensit(cpl_table *, cpl_table *, 
        const char *, int) ;
static int crires_util_plot_spec_wl_one(cpl_table *, cpl_table *, 
        const char *, int, const char *, const char *, const char *) ;

static char crires_util_plot_description[] = "Plot the CRIRES tables.\n" 
"For MEF, specify the extension with --display=extension_number.\n"
"This recipe accepts possibly 2 parameter:\n"
"First parameter:       the table to plot.\n"
"                       (PRO TYPE = "CRIRES_PROTYPE_CATALOG") or\n"
"                       (PRO TYPE = "CRIRES_PROTYPE_PHO_FLUX") or\n"
"                       (PRO TYPE = "CRIRES_PROTYPE_SPEC_PIX") or\n"
"                       (PRO TYPE = "CRIRES_PROTYPE_SPEC_WL") or\n"
"                       (PRO TYPE = "CRIRES_PROTYPE_XCORR") or\n"
"                       (PRO TYPE = "CRIRES_PROTYPE_CONVERS") or\n"
"                       (PRO TYPE = "CRIRES_PROTYPE_SENSIT") or\n"
"                       (PRO TYPE = "CRIRES_PROTYPE_FWHM")\n" 
"Second parameter is optional and must be of the same type and same\n"
"                 table length as the first one. If provided, the two\n"
"                 signals are overplotted. In this case, --adjust can \n"
"                 be used to adjust the second plot average level to \n"
"                 the first one.\n" ;

CRIRES_RECIPE_DEFINE(crires_util_plot,
        CRIRES_PARAM_PMIN       |
        CRIRES_PARAM_PMAX       |
        CRIRES_PARAM_DISPLAY    |
        CRIRES_PARAM_WL_MODEL   |
        CRIRES_PARAM_ADJUST,
        "Plotting tool for the CRIRES tables",
        crires_util_plot_description) ;

/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static struct {
    /* Inputs */
    int             pmin ;
    int             pmax ;
    int             chip ;
    int             use_model ;
    int             adjust_level ;
} crires_util_plot_config ;

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    frameset   the frames list
  @param    parlist     the parameters list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_util_plot(
        cpl_frameset            *   frameset,
        const cpl_parameterlist *   parlist)
{
    const char          *   fname ;
    const char          *   fname_opt ;
    const char          *   title ;
    const char          *   sval ;
    cpl_propertylist    *   plist ;
    cpl_table           *   tab ;
    cpl_table           *   tab_opt ;
    cpl_array           *   wave ;
    cpl_array           *   flux ;
    cpl_vector          *   wave_vec ;
    cpl_vector          *   flux_vec ;
    cpl_bivector        *   flux_biv ;
    const char          *   wave_col ;
    int                     i ;
   
    /* Retrieve input parameters */
    crires_util_plot_config.pmin = crires_parameterlist_get_int(parlist,
            RECIPE_STRING, CRIRES_PARAM_PMIN) ;
    crires_util_plot_config.pmax = crires_parameterlist_get_int(parlist,
            RECIPE_STRING, CRIRES_PARAM_PMAX) ;
    crires_util_plot_config.chip = crires_parameterlist_get_int(parlist,
            RECIPE_STRING, CRIRES_PARAM_DISPLAY) ;
    crires_util_plot_config.use_model = crires_parameterlist_get_bool(parlist, 
            RECIPE_STRING, CRIRES_PARAM_WL_MODEL) ;
    crires_util_plot_config.adjust_level=crires_parameterlist_get_bool(parlist, 
            RECIPE_STRING, CRIRES_PARAM_ADJUST) ;

    /* Initialise */
    if (crires_util_plot_config.use_model)  
        wave_col = CRIRES_COL_WAVELENGTH_MODEL ;
    else
        wave_col = CRIRES_COL_WAVELENGTH ;
    
    /* Identify the RAW and CALIB frames in the input frameset */
    if (crires_dfs_set_groups(frameset, NULL)) {
        cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
        return -1 ;
    }

    /* Retrieve raw frames */
    fname = cpl_frame_get_filename(cpl_frameset_get_position(frameset, 0)) ;
    if (cpl_frameset_get_size(frameset) > 1) {
        fname_opt = cpl_frame_get_filename(
                cpl_frameset_get_position(frameset, 1)) ;
    } else {
        fname_opt = NULL ;
    }

    /* CRIRES_PROTYPE_CATALOG */
    tab = crires_load_table_check(fname, 1, CRIRES_PROTYPE_CATALOG, 
            crires_util_plot_config.pmin,
            crires_util_plot_config.pmax,
            crires_util_plot_config.use_model) ;
    if (tab != NULL) {
        plist = cpl_propertylist_load(fname, 0) ;
        sval = crires_pfits_get_procatg(plist) ;
        /* CRIRES_CALPRO_THAR_CAT */
        if (!strcmp(sval, CRIRES_CALPRO_THAR_CAT))
            title = "t 'Thorium/Argon lines' w lines" ;
        /* CRIRES_CALPRO_N2O_CAT */
        else if (!strcmp(sval, CRIRES_CALPRO_N2O_CAT))  
            title = "t 'N2O gas cell lines' w lines" ;
        /* CRIRES_CALPRO_OH_CAT */
        else if (!strcmp(sval, CRIRES_CALPRO_OH_CAT))  
            title = "t 'OH lines' w lines" ;
        /* CRIRES_CALPRO_HITRAN_CAT */
        else if (!strcmp(sval, CRIRES_CALPRO_HITRAN_CAT))  
            title = "t 'Hitran Spectrum' w lines" ;
        /* Default */
        else        
            title = "t 'signal' w lines" ;
        cpl_propertylist_delete(plist) ;

        /* Plot */
        cpl_plot_column(
                "set grid;set xlabel 'Wavelength (nm)';set ylabel 'Emission';",
                title, "", tab, CRIRES_COL_WAVELENGTH, CRIRES_COL_EMISSION) ;
        cpl_table_delete(tab) ;
    }
 
    /* CRIRES_PROTYPE_PHO_FLUX */
    tab = crires_load_table_check(fname, 1, CRIRES_PROTYPE_PHO_FLUX, 
            crires_util_plot_config.pmin,
            crires_util_plot_config.pmax,
            crires_util_plot_config.use_model) ;
    if (tab != NULL) {
        for (i=1 ; i<cpl_table_get_nrow(tab) ; i++) {
            wave=cpl_array_duplicate(
                    cpl_table_get_array(tab, CRIRES_COL_PHOTOFLUX, 0));
            wave_vec = cpl_vector_wrap(cpl_array_get_size(wave),
                    cpl_array_get_data_double(wave)) ;
            cpl_array_unwrap(wave) ;
            flux=cpl_array_duplicate(
                    cpl_table_get_array(tab, CRIRES_COL_PHOTOFLUX, i));
            flux_vec = cpl_vector_wrap(cpl_array_get_size(flux),
                    cpl_array_get_data_double(flux)) ;
            cpl_array_unwrap(flux) ;
            flux_biv=cpl_bivector_wrap_vectors(wave_vec, flux_vec) ;

            cpl_plot_bivector(
                "set grid;set xlabel 'Wavelength (nm)';set ylabel 'Flux (Jy)';",
                "t 'Photspheric flux' w lines", "", flux_biv) ;
            cpl_bivector_delete(flux_biv) ;
        }
        cpl_table_delete(tab) ;
    }
    /* CRIRES_PROTYPE_SPEC_PIX */
    tab = crires_load_table_check(fname, crires_util_plot_config.chip, 
            CRIRES_PROTYPE_SPEC_PIX, crires_util_plot_config.pmin, 
            crires_util_plot_config.pmax, crires_util_plot_config.use_model) ;
    if (fname_opt != NULL) {
        tab_opt = crires_load_table_check(fname_opt, 
                crires_util_plot_config.chip, CRIRES_PROTYPE_SPEC_PIX, 
                crires_util_plot_config.pmin, crires_util_plot_config.pmax, 
                crires_util_plot_config.use_model);
    } else {
        tab_opt = NULL ;
    }
    if (tab != NULL) {
        crires_util_plot_spec_pix(tab, tab_opt,
                crires_util_plot_config.adjust_level) ;
        cpl_table_delete(tab) ;
        if (tab_opt != NULL) cpl_table_delete(tab_opt) ;
    } 

    /* CRIRES_PROTYPE_SPEC_WL */
    tab = crires_load_table_check(fname, crires_util_plot_config.chip, 
            CRIRES_PROTYPE_SPEC_WL, crires_util_plot_config.pmin, 
            crires_util_plot_config.pmax, crires_util_plot_config.use_model) ;
    if (fname_opt != NULL) {
        tab_opt = crires_load_table_check(fname_opt, 
                crires_util_plot_config.chip, CRIRES_PROTYPE_SPEC_WL, 
                crires_util_plot_config.pmin, crires_util_plot_config.pmax, 
                crires_util_plot_config.use_model);
    } else {
        tab_opt = NULL ;
    }
    if (tab != NULL) {
        crires_util_plot_spec_wl(tab, tab_opt, wave_col,
                crires_util_plot_config.adjust_level) ;
        cpl_table_delete(tab) ;
        if (tab_opt != NULL) cpl_table_delete(tab_opt) ;
    }

   /* CRIRES_PROTYPE_XCORR */
    tab = crires_load_table_check(fname, crires_util_plot_config.chip, 
            CRIRES_PROTYPE_XCORR, 
            crires_util_plot_config.pmin,
            crires_util_plot_config.pmax,
            crires_util_plot_config.use_model) ;
    if (tab != NULL) {
        irplib_wlxcorr_plot_spc_table(tab, "", 1, 5) ;
    }

    /* CRIRES_PROTYPE_CONVERS */
    tab = crires_load_table_check(fname, crires_util_plot_config.chip, 
            CRIRES_PROTYPE_CONVERS, crires_util_plot_config.pmin, 
            crires_util_plot_config.pmax, crires_util_plot_config.use_model) ;
    if (fname_opt != NULL) {
        tab_opt = crires_load_table_check(fname_opt, 
                crires_util_plot_config.chip, CRIRES_PROTYPE_CONVERS, 
                crires_util_plot_config.pmin, crires_util_plot_config.pmax, 
                crires_util_plot_config.use_model);
    } else {
        tab_opt = NULL ;
    }
    if (tab != NULL) {
        crires_util_plot_convers(tab, tab_opt, wave_col,
                crires_util_plot_config.adjust_level) ;
        cpl_table_delete(tab) ;
        if (tab_opt != NULL) cpl_table_delete(tab_opt) ;
    }

    /* CRIRES_PROTYPE_SENSIT */
    tab = crires_load_table_check(fname, crires_util_plot_config.chip, 
            CRIRES_PROTYPE_SENSIT, crires_util_plot_config.pmin, 
            crires_util_plot_config.pmax, crires_util_plot_config.use_model) ;
    if (fname_opt != NULL) {
        tab_opt = crires_load_table_check(fname_opt, 
                crires_util_plot_config.chip, CRIRES_PROTYPE_SENSIT, 
                crires_util_plot_config.pmin, crires_util_plot_config.pmax, 
                crires_util_plot_config.use_model);
    } else {
        tab_opt = NULL ;
    }
    if (tab != NULL) {
        crires_util_plot_sensit(tab, tab_opt, wave_col,
                crires_util_plot_config.adjust_level) ;
        cpl_table_delete(tab) ;
        if (tab_opt != NULL) cpl_table_delete(tab_opt) ;
    }

    /* CRIRES_PROTYPE_FWHM */
    tab = crires_load_table_check(fname, crires_util_plot_config.chip, 
            CRIRES_PROTYPE_FWHM, crires_util_plot_config.pmin, 
            crires_util_plot_config.pmax, crires_util_plot_config.use_model) ;
    if (tab != NULL) {
        cpl_plot_column(
        "set grid;set xlabel 'Position (pixels)';set ylabel 'FWHM (pixels)';",
                "t 'FWHM' w lines", "", tab,
                CRIRES_COL_WL_DETPOSX, CRIRES_COL_FWHM) ;
        cpl_plot_column(
    "set grid;set xlabel 'Position (pixels)';set ylabel 'Y position (pixels)';",
                "t 'Y position' w lines", "", tab,
                CRIRES_COL_WL_DETPOSX, CRIRES_COL_DETPOSY) ;
        cpl_table_delete(tab) ;
    }
 
    /* Return */
    if (cpl_error_get_code()) 
        return -1 ;
    else 
        return 0 ;
}

static int crires_util_plot_diff(
        const cpl_table *       tab,
        const char      *       col1,
        const char      *       col2)
{
    cpl_bivector    *   toplot ;
    cpl_vector      *   vec_x ;
    cpl_vector      *   vec_tmp ;
    cpl_vector      *   vec_y ;
    int                 nrows ;
    double              val ;
    int                 i ;

    /* Check entries */
    if (tab == NULL) return -1 ;
    if (!cpl_table_has_column(tab, col1)) return -1 ;
    if (!cpl_table_has_column(tab, col2)) return -1 ;

    /* Initialise */
    nrows = cpl_table_get_nrow(tab) ;

    /* Get access */
    vec_x = cpl_vector_wrap(nrows,
            cpl_table_get_data_double((cpl_table*)tab, col1)) ;
    vec_tmp = cpl_vector_wrap(nrows,
            cpl_table_get_data_double((cpl_table*)tab, col2)) ;

    /* Compute difference */
    vec_y = cpl_vector_duplicate(vec_x) ;
    cpl_vector_subtract(vec_y, vec_tmp) ;
    cpl_vector_unwrap(vec_tmp) ;

    /* Handle special case 0s */
    for (i=0 ; i<cpl_vector_get_size(vec_y) ; i++) {
        if (fabs(cpl_vector_get(vec_y, i)) > 100) {
            if (i==0) val = cpl_vector_get(vec_y, 10) ;
            else val = cpl_vector_get(vec_y, i-1) ;
            cpl_vector_set(vec_y, i, val) ; 
        }
    }

    toplot = cpl_bivector_wrap_vectors(vec_x, vec_y) ;

    /* Plot */
    cpl_plot_bivector(
        "set grid;set xlabel 'Wavelength (nm)';set ylabel 'Wave diff (nm)';",
        "t 'Wavelength difference XC-model' w lines", "", toplot) ;

    /* Free and return */
    cpl_bivector_unwrap_vectors(toplot) ;
    cpl_vector_unwrap(vec_x) ;
    cpl_vector_delete(vec_y) ;

    return 0 ;
}

static int crires_util_plot_spec_pix(
        cpl_table   *       tab,
        cpl_table   *       tab_opt,
        int                 adjust_level)
{
    /* Protect empty chips in windowing mode */
    if (cpl_table_get_nrow(tab) == 0) {
        return 0 ;
    }

    /* CRIRES_COL_EXTRACT_INT_OPT */
    crires_util_plot_spec_pix_one(tab, tab_opt, adjust_level, 
            CRIRES_COL_EXTRACT_INT_OPT,
"set grid;set xlabel 'Position (pixels)';set ylabel 'Intensity OPT (ADU/sec)';",
            "t 'Extracted Spectrum OPT' w lines") ;

    /* CRIRES_COL_EXTRACT_INT_RECT */
    crires_util_plot_spec_pix_one(tab, tab_opt, adjust_level, 
            CRIRES_COL_EXTRACT_INT_RECT,
"set grid;set xlabel 'Position (pixels)';set ylabel 'Intensity RECT (ADU/sec)';",
            "t 'Extracted Spectrum RECT' w lines") ;

    /* CRIRES_COL_EXTRACT_ERR_OPT */
    crires_util_plot_spec_pix_one(tab, tab_opt, adjust_level, 
            CRIRES_COL_EXTRACT_ERR_OPT,
"set grid;set xlabel 'Position (pixels)';set ylabel 'Error OPT (ADU/sec)';",
            "t 'Error OPT' w lines") ;

    /* CRIRES_COL_EXTRACT_ERR_RECT */
    crires_util_plot_spec_pix_one(tab, tab_opt, adjust_level, 
            CRIRES_COL_EXTRACT_ERR_RECT,
"set grid;set xlabel 'Position (pixels)';set ylabel 'Error RECT (ADU/sec)';",
            "t 'Error RECT' w lines") ;

    /* CRIRES_COL_EXTRACT_WN_OPT */
    crires_util_plot_spec_pix_one(tab, tab_opt, adjust_level, 
            CRIRES_COL_EXTRACT_WN_OPT,
"set grid;set xlabel 'Position (pixels)';set ylabel 'Weight Norm OPT';", 
            "t 'Weight norm OPT' w lines") ;

    /* CRIRES_COL_EXTRACT_WN_RECT */
    crires_util_plot_spec_pix_one(tab, tab_opt, adjust_level, 
            CRIRES_COL_EXTRACT_WN_RECT,
"set grid;set xlabel 'position (pixels)';set ylabel 'weight norm rect';", 
            "t 'weight norm rect' w lines") ;

    /* CRIRES_COL_EXTRACT_BG_NOISE */
    crires_util_plot_spec_pix_one(tab, tab_opt, adjust_level, 
            CRIRES_COL_EXTRACT_BG_NOISE,
"set grid;set xlabel 'Position (pixels)';set ylabel 'Background noise';", 
            "t 'Background noise' w lines") ;

    return 0 ;
}
 
static int crires_util_plot_spec_pix_one(
        cpl_table   *       tab,
        cpl_table   *       tab_opt,
        int                 adjust_level,
        const char  *       y_col,
        const char  *       options,
        const char  *       title)
{
    double                  mean1, mean2 ;
    int                     nrows ;
    cpl_vector          **  vectors ;

    /* Check inputs */
    if (tab == NULL) return -1 ;
    nrows = cpl_table_get_nrow(tab) ;
    if (tab_opt != NULL) {
        if (cpl_table_get_nrow(tab_opt) != nrows) {
            cpl_msg_error(__func__, 
                    "The two tables must have the same number of rows") ;
            return -1 ;
        }
    }
    
    vectors = cpl_malloc(3*sizeof(cpl_vector*)) ;
    vectors[0] = NULL ;
    vectors[1] = cpl_vector_wrap(nrows, cpl_table_get_data_double(tab, y_col)) ;
    if (tab_opt != NULL) {
        vectors[2] = cpl_vector_wrap(nrows,
                cpl_table_get_data_double(tab_opt, y_col));
        if (adjust_level) {
            mean1 = cpl_vector_get_mean(vectors[1]) ;
            mean2 = cpl_vector_get_mean(vectors[2]) ;
            cpl_vector_multiply_scalar(vectors[2], fabs(mean1/mean2)) ;
        }
        cpl_plot_vectors(options, title, "", (const cpl_vector **)vectors, 3);
        cpl_vector_unwrap(vectors[2]) ;
    } else {
        cpl_plot_vector(options, title, "", vectors[1]) ;
    }
    cpl_vector_unwrap(vectors[1]) ;
    cpl_free(vectors) ;
    return 0 ;
}

static int crires_util_plot_spec_wl(
        cpl_table   *       tab,
        cpl_table   *       tab_opt,
        const char  *       wave_col,
        int                 adjust_level)
{
    /* Protect empty chips in windowing mode */
    if (cpl_table_get_nrow(tab) == 0) {
        return 0 ;
    }

    /* CRIRES_COL_EXTRACT_INT_OPT */
    crires_util_plot_spec_wl_one(tab, tab_opt, wave_col, adjust_level, 
            CRIRES_COL_EXTRACT_INT_OPT,
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Intensity OPT (ADU/sec)';",
            "t 'Extracted Spectrum OPT' w lines") ;

    /* CRIRES_COL_EXTRACT_INT_RECT */
    crires_util_plot_spec_wl_one(tab, tab_opt, wave_col, adjust_level, 
            CRIRES_COL_EXTRACT_INT_RECT,
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Intensity RECT (ADU/sec)';",
            "t 'Extracted Spectrum RECT' w lines") ;

    /* CRIRES_COL_EXTRACT_ERR_OPT */
    crires_util_plot_spec_wl_one(tab, tab_opt, wave_col, adjust_level, 
            CRIRES_COL_EXTRACT_ERR_OPT,
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Error OPT (ADU/sec)';",
            "t 'Error OPT' w lines") ;

    /* CRIRES_COL_EXTRACT_ERR_RECT */
    crires_util_plot_spec_wl_one(tab, tab_opt, wave_col, adjust_level, 
            CRIRES_COL_EXTRACT_ERR_RECT,
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Error RECT (ADU/sec)';",
            "t 'Error RECT' w lines") ;

    /* CRIRES_COL_EXTRACT_WN_OPT */
    crires_util_plot_spec_wl_one(tab, tab_opt, wave_col, adjust_level, 
            CRIRES_COL_EXTRACT_WN_OPT,
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Weight Norm OPT';",
            "t 'Weight Norm OPT' w lines") ;

    /* CRIRES_COL_EXTRACT_WN_RECT */
    crires_util_plot_spec_wl_one(tab, tab_opt, wave_col, adjust_level, 
            CRIRES_COL_EXTRACT_WN_RECT,
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Weight Norm RECT';",
            "t 'Weight Norm RECT' w lines") ;

    /* CRIRES_COL_EXTRACT_BG_NOISE */
    crires_util_plot_spec_wl_one(tab, tab_opt, wave_col, adjust_level, 
            CRIRES_COL_EXTRACT_BG_NOISE,
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Background noise';",
            "t 'Background noise' w lines") ;

    /* Wavelengths difference */
    crires_util_plot_diff(tab, CRIRES_COL_WAVELENGTH,
            CRIRES_COL_WAVELENGTH_MODEL) ;
    
    return 0 ;
}
 
static int crires_util_plot_spec_wl_one(
        cpl_table   *       tab,
        cpl_table   *       tab_opt,
        const char  *       wave_col,
        int                 adjust_level,
        const char  *       y_col,
        const char  *       options,
        const char  *       title)
{
    double                  mean1, mean2 ;
    int                     nrows ;
    cpl_vector          **  vectors ;

    /* Check inputs */
    if (tab == NULL) return -1 ;
    nrows = cpl_table_get_nrow(tab) ;
    if (tab_opt != NULL) {
        if (cpl_table_get_nrow(tab_opt) != nrows) {
            cpl_msg_error(__func__, 
                    "The two tables must have the same number of rows") ;
            return -1 ;
        }
    }

    if (tab_opt != NULL) {
        vectors = cpl_malloc(3*sizeof(cpl_vector*)) ;
        vectors[0]=cpl_vector_wrap(nrows,cpl_table_get_data_double(tab, 
                    wave_col));
        vectors[1] = cpl_vector_wrap(nrows, 
                cpl_table_get_data_double(tab, y_col)) ;
        vectors[2] = cpl_vector_wrap(nrows,
                cpl_table_get_data_double(tab_opt, y_col));
        if (adjust_level) {
            mean1 = cpl_vector_get_mean(vectors[1]) ;
            mean2 = cpl_vector_get_mean(vectors[2]) ;
            cpl_vector_multiply_scalar(vectors[2], fabs(mean1/mean2)) ;
        }
        cpl_plot_vectors(options, title, "", (const cpl_vector **)vectors, 3);
        cpl_vector_unwrap(vectors[0]) ;
        cpl_vector_unwrap(vectors[1]) ;
        cpl_vector_unwrap(vectors[2]) ;
        cpl_free(vectors) ;
    } else {
        cpl_plot_column(options, title, "", tab, wave_col, y_col) ;
    }
    return 0 ;
}

static int crires_util_plot_convers(
        cpl_table   *       tab,
        cpl_table   *       tab_opt,
        const char  *       wave_col,
        int                 adjust_level)
{
    /* Protect empty chips in windowing mode */
    if (cpl_table_get_nrow(tab) == 0) {
        return 0 ;
    }

    /* Start with the same as for CRIRES_PROTYPE_SPEC_WL */
    if (crires_util_plot_spec_wl(tab, tab_opt, wave_col, adjust_level) == -1) 
        return -1 ;
    
    /* CRIRES_COL_CONVERSION_OPT  */
    crires_util_plot_spec_wl_one(tab, tab_opt, wave_col, adjust_level, 
            CRIRES_COL_CONVERSION_OPT,
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Conversion OPT (ADU/sec/Jy)';",
        "t 'Conversion factor OPT' w lines") ;

    /* CRIRES_COL_CONVERSION_RECT */
    crires_util_plot_spec_wl_one(tab, tab_opt, wave_col, adjust_level, 
            CRIRES_COL_CONVERSION_RECT,
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Conversion RECT (ADU/sec/Jy)';",
        "t 'Conversion factor RECT' w lines") ;

    /* CRIRES_COL_THROUGHPUT */
    crires_util_plot_spec_wl_one(tab, tab_opt, wave_col, adjust_level, 
            CRIRES_COL_THROUGHPUT,
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Throughput (e-/phot)';",
        "t 'Throughput' w lines") ;
    return 0 ;
}

static int crires_util_plot_sensit(
        cpl_table   *       tab,
        cpl_table   *       tab_opt,
        const char  *       wave_col,
        int                 adjust_level)
{
    /* Protect empty chips in windowing mode */
    if (cpl_table_get_nrow(tab) == 0) {
        return 0 ;
    }

    /* Start with the same as for CRIRES_PROTYPE_CONVERS */
    if (crires_util_plot_convers(tab, tab_opt, wave_col, adjust_level) == -1) 
        return -1 ;
    
    /* CRIRES_COL_SENSITIVITY */
    crires_util_plot_spec_wl_one(tab, tab_opt, wave_col, adjust_level, 
            CRIRES_COL_SENSITIVITY,
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Sensitivity (Jy/10sig/hour)';",
                "t 'Sensitivity' w lines") ;
    return 0 ;
}

