/* $Id: crires_util_wlassign.c,v 1.37 2012-09-20 13:09:54 yjung Exp $
 *
 * This file is part of the crires Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: yjung $
 * $Date: 2012-09-20 13:09:54 $
 * $Revision: 1.37 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include "crires_recipe.h"
#include "crires_wlcalib.h"

/*-----------------------------------------------------------------------------
                                Define
 -----------------------------------------------------------------------------*/

#define RECIPE_STRING "crires_util_wlassign"

/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static int crires_util_wlassign_save(const cpl_table **, 
        const cpl_parameterlist *, cpl_frameset *) ;

static char crires_util_wlassign_description[] = 
"This recipe accepts 2 parameters:\n"
"First parameter:   the table of the extracted spectrum in pixels\n"
"                   (PRO TYPE = "CRIRES_PROTYPE_SPEC_PIX")\n"
"Second parameter:  the table with the wavelength solution(s)\n"
"                   (PRO TYPE = "CRIRES_PROTYPE_WL_POLY")\n"
"\n"
"This recipe produces 1 file:\n"
"First product:     the table with the extracted spectrum in wavelength.\n"
"                   (PRO TYPE = "CRIRES_PROTYPE_SPEC_WL")\n" ;

CRIRES_RECIPE_DEFINE(crires_util_wlassign,
        CRIRES_PARAM_DISPLAY,
        "Put the wavelength in the extracted table",
        crires_util_wlassign_description) ;

/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static struct {
    /* Inputs */
    int             display ;
    /* Outputs */
    int             win_mode ;
} crires_util_wlassign_config ;

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    frameset   the frames list
  @param    parlist     the parameters list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_util_wlassign(
        cpl_frameset            *   frameset,
        const cpl_parameterlist *   parlist)
{
    cpl_frame           *   fr ;
    cpl_propertylist    *   plist ;
    const char          *   sval ;
    cpl_table           *   wl_tab ;
    cpl_image           *   wl_map ;
    cpl_table           *   ext_tab[CRIRES_NB_DETECTORS] ;
    int                     nbrows, pix, spec_pos ;
    double                  wl ;
    int                     i, j ;
   
    /* Initialise */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        ext_tab[i] = NULL ;
    }

    /* Retrieve input parameters */
    crires_util_wlassign_config.display = crires_parameterlist_get_int(parlist,
            RECIPE_STRING, CRIRES_PARAM_DISPLAY) ;
    
    /* Identify the RAW and CALIB frames in the input frameset */
    if (crires_dfs_set_groups(frameset, NULL)) {
        cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
        return -1 ;
    }

    /* Windowing mode ? */
    fr = cpl_frameset_get_position(frameset, 0);
    if ((plist=cpl_propertylist_load(cpl_frame_get_filename(fr), 0)) == NULL)
        return -1 ;
    sval = crires_pfits_get_ncorrs(plist) ;
    if (!strcmp(sval, "FowlerNsampGRstWin")) {
        crires_util_wlassign_config.win_mode = 1 ;
    } else {
        crires_util_wlassign_config.win_mode = 0 ;
    }
    cpl_propertylist_delete(plist) ;

    /* Loop on the detectors */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        /* Skip some detectors in windowing mode */
        if ((i==0 || i==CRIRES_NB_DETECTORS-1) &&
                (crires_util_wlassign_config.win_mode == 1)) {
            continue ;
        }
        cpl_msg_info(__func__, "Wavelength assignment for chip %d", i+1) ;

        /* First Read the Wavelength calibration from the second frame */
        cpl_msg_info(__func__, "Wavelength retrieval") ;
        fr = cpl_frameset_get_position(frameset, 1);
        if ((wl_tab=crires_load_table_check(cpl_frame_get_filename(fr), i+1, 
                        CRIRES_PROTYPE_WL_POLY, -1, -1, 0)) == NULL) {
            cpl_msg_error(__func__, "Cannot load Wavelength from chip %d",i+1) ;
            return -1 ;
        }
    
        /* Create the wave map */
        if ((wl_map = crires_wlcalib_gen_wlmap_one_chip(
                        (const cpl_table *)wl_tab)) == NULL) {
            cpl_msg_error(__func__, "Cannot compute the Wavelength Map") ;
            cpl_table_delete(wl_tab);
            return -1 ;
        }
        cpl_table_delete(wl_tab);

        /* The first frame must be the extracted spectrum in pixels */
        fr = cpl_frameset_get_position(frameset, 0);
        if ((ext_tab[i] = crires_load_table_check(cpl_frame_get_filename(fr), 
                        i+1, CRIRES_PROTYPE_SPEC_PIX, -1, -1, 0)) == NULL) {
            cpl_msg_warning(__func__, "Empty extension") ;
            continue ;
        }

        /* Create the output table */
        cpl_msg_info(__func__, "Wavelength column computation") ;
        cpl_table_new_column(ext_tab[i], CRIRES_COL_WAVELENGTH,CPL_TYPE_DOUBLE);
        cpl_table_new_column(ext_tab[i], CRIRES_COL_WAVELENGTH_MODEL,
                CPL_TYPE_DOUBLE);
        nbrows = cpl_table_get_nrow(ext_tab[i]) ;
        spec_pos = crires_get_y_spec_position(cpl_frame_get_filename(fr), i+1);
        for (j=0 ; j<nbrows ; j++) {
            wl = cpl_image_get(wl_map, j+1, spec_pos, &pix) ;
            cpl_table_set_double(ext_tab[i], CRIRES_COL_WAVELENGTH, j, wl);
            cpl_table_set_double(ext_tab[i], CRIRES_COL_WAVELENGTH_MODEL, j, 
                    0.0);
        }
        cpl_image_delete(wl_map) ;

        /* Plot if requested */
        if(crires_util_wlassign_config.display == i+1) {
            cpl_plot_column(
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Intensity OPT (ADU/sec)';",
                    "t 'Extracted spectrum OPT' w lines", "", ext_tab[i],
                    CRIRES_COL_WAVELENGTH, CRIRES_COL_EXTRACT_INT_OPT) ;
            cpl_plot_column(
"set grid;set xlabel 'Wavelength (nm)';set ylabel 'Intensity RECT (ADU/sec)';",
                    "t 'Extracted spectrum RECT' w lines", "", ext_tab[i],
                    CRIRES_COL_WAVELENGTH, CRIRES_COL_EXTRACT_INT_RECT) ;
        }
    }
   
    /* Reconstruct chips in windowing mode using detector 2 */
    if (crires_util_wlassign_config.win_mode == 1) {
        if (ext_tab[1] != NULL) {
            ext_tab[0] = cpl_table_duplicate(ext_tab[1]) ;
            cpl_table_set_size(ext_tab[0], 0) ;
            ext_tab[CRIRES_NB_DETECTORS-1] = cpl_table_duplicate(ext_tab[0]) ;
        }
    }

    /* Save the product */
    cpl_msg_info(__func__, "Save the product") ;
    cpl_msg_indent_more() ;
    if (crires_util_wlassign_save((const cpl_table **)ext_tab, parlist, 
                frameset)) {
        cpl_msg_error(__func__, "Cannot save the product") ;
        for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
            if (ext_tab[i] != NULL) cpl_table_delete(ext_tab[i]) ;
        }
        cpl_msg_indent_less() ;
        return -1 ;
    }
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        if (ext_tab[i] != NULL) cpl_table_delete(ext_tab[i]) ;
    }
    cpl_msg_indent_less() ;

    /* Return */
    if (cpl_error_get_code()) return -1 ;
    else return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Save the products
  @param    out         the tables to save
  @param    parlist     the parameters list
  @param    set         the frames set
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_util_wlassign_save(
        const cpl_table         **  out,
        const cpl_parameterlist *   parlist,
        cpl_frameset            *   set)
{
    const char          *   recipe_name = "crires_util_wlassign" ;

    /* Write the table */
    crires_table_save(set,
            parlist,
            set,
            out,
            recipe_name,
            CRIRES_OBS_EXTRACT_WL_TAB,
            CRIRES_PROTYPE_SPEC_WL,
            NULL,
            NULL,
            PACKAGE "/" PACKAGE_VERSION,
            "crires_util_wlassign.fits") ;

    return  0;
}
