/* $Id: crires_util_wlinit.c,v 1.39 2012-09-20 12:06:34 yjung Exp $
 *
 * This file is part of the crires Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: yjung $
 * $Date: 2012-09-20 12:06:34 $
 * $Revision: 1.39 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include <locale.h>
#include "crires_recipe.h"

#include "crires_wlcalib.h"
#include "crires_wlestimate.h"
#include "irplib_utils.h"

/*-----------------------------------------------------------------------------
                                Define
 -----------------------------------------------------------------------------*/

#define RECIPE_STRING "crires_util_wlinit"

/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static int crires_util_wlinit_save(const cpl_imagelist *, const cpl_table **, 
        const cpl_parameterlist *, cpl_frameset *) ;

static char crires_util_wlinit_description[] = 
"This recipe accepts 1 parameters:\n"
"First parameter:   Any FITS file containing the WLEN settings in its header.\n"
"\n"
"This recipe produces 2 files:\n"
"First product:     the image with the wavelength values.\n"
"                   (PRO TYPE = "CRIRES_PROTYPE_WL_MAP")\n"
"Second product:    the table with the wavelength polynomial coefficients.\n"
"                   (PRO TYPE = "CRIRES_PROTYPE_WL_POLY")\n" ;

CRIRES_RECIPE_DEFINE(crires_util_wlinit,
        CRIRES_PARAM_WAVES,
        "Wavelength calibration from the initial guess",
        crires_util_wlinit_description) ;

/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static struct {
    /* Inputs */
    double                  wmin[CRIRES_NB_DETECTORS] ;
    double                  wmax[CRIRES_NB_DETECTORS] ;
    /* Outputs */
    crires_illum_period     period ;
    double                  qc_wlcent[CRIRES_NB_DETECTORS] ;
    double                  qc_wldisp[CRIRES_NB_DETECTORS] ;
} crires_util_wlinit_config ;

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    frameset   the frames list
  @param    parlist     the parameters list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_util_wlinit(
        cpl_frameset            *   frameset,
        const cpl_parameterlist *   parlist)
{
    const char          *   sval ;
    cpl_imagelist       *   wl_map ;
    cpl_table           *   wl_tab[CRIRES_NB_DETECTORS] ;
    cpl_polynomial      *   pol[CRIRES_NB_DETECTORS] ;
    int                     pix ;
    int                     i, j ;

    /* Needed for sscanf() */
    setlocale(LC_NUMERIC, "C");
    
    /* Retrieve input parameters */
    sval = crires_parameterlist_get_string(parlist, RECIPE_STRING,
            CRIRES_PARAM_WAVES) ;
    if (sscanf(sval, "%lg,%lg,%lg,%lg,%lg,%lg,%lg,%lg",
                    &crires_util_wlinit_config.wmin[0],
                    &crires_util_wlinit_config.wmax[0],
                    &crires_util_wlinit_config.wmin[1],
                    &crires_util_wlinit_config.wmax[1],
                    &crires_util_wlinit_config.wmin[2],
                    &crires_util_wlinit_config.wmax[2],
                    &crires_util_wlinit_config.wmin[3],
                    &crires_util_wlinit_config.wmax[3])!=2*CRIRES_NB_DETECTORS){
        return -1 ;
    }

    /* Identify the RAW and CALIB frames in the input frameset */
    if (crires_dfs_set_groups(frameset, NULL)) {
        cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
        return -1 ;
    }

    /* Get the detector illumination period */
    crires_util_wlinit_config.period =
        crires_get_detector_illum_period(
                cpl_frame_get_filename(cpl_frameset_get_position(frameset, 0)));
    if (crires_util_wlinit_config.period == CRIRES_ILLUM_UNKNOWN) {
        cpl_msg_error(__func__,
                "Cannot determine the detector illumination period") ;
        return -1 ;
    } else {
        crires_display_detector_illum(crires_util_wlinit_config.period) ;
    }

    /* Loop on the spectra to calibrate */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        cpl_msg_info(__func__, "Wavelength calibration for chip %d", i+1) ;

        /* Initialise */
        crires_util_wlinit_config.qc_wlcent[i] = -1.0 ;
        crires_util_wlinit_config.qc_wldisp[i] = -1.0 ;

        /* Get the wavelength estimate */
        if ((pol[i] = crires_wlestimate_compute(
                        crires_util_wlinit_config.wmin[i],
                        crires_util_wlinit_config.wmax[i])) == NULL) {
            if ((pol[i] = crires_wlestimate_get(cpl_frame_get_filename(
                        cpl_frameset_get_position(frameset,0)), i+1)) == NULL) {
                cpl_msg_error(__func__, "Cannot get the calibration") ;
                cpl_msg_indent_less() ;
                for (j=0 ; j<i ; j++) cpl_polynomial_delete(pol[j]) ;
                return -1 ;
            }
        }

        /* Create the table */
        wl_tab[i] = crires_wlcalib_gen_wltab_one(
                (const cpl_polynomial *)pol[i], -1, -1.0) ;
        if (pol[i] != NULL) cpl_polynomial_delete(pol[i]) ;
    }
    cpl_msg_indent_less() ;

    /* Create the map */
    wl_map = crires_wlcalib_gen_wlmap((const cpl_table **)wl_tab) ;
    
    /* Compute the QC parameters */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        crires_util_wlinit_config.qc_wlcent[i] =
            cpl_image_get(cpl_imagelist_get(wl_map, i),
                    512, 256, &pix) ;
        crires_util_wlinit_config.qc_wldisp[i] =
            ((cpl_image_get(cpl_imagelist_get(wl_map, i), 1024, 256, &pix)) -
             (cpl_image_get(cpl_imagelist_get(wl_map, i), 1, 256, &pix)))
            / 1023 ;
    }
    
    /* Save the products */
    cpl_msg_info(__func__, "Save the products") ;
    cpl_msg_indent_more() ;
    if (crires_util_wlinit_save(wl_map, (const cpl_table **)wl_tab, parlist, 
                frameset)) {
        cpl_msg_error(__func__, "Cannot save the products") ;
        cpl_msg_indent_less() ;
        cpl_imagelist_delete(wl_map) ;
        for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
            if (wl_tab[i] != NULL) cpl_table_delete(wl_tab[i]) ;
        }
        return -1 ;
    }
    cpl_msg_indent_less() ;
    cpl_imagelist_delete(wl_map) ;
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        if (wl_tab[i] != NULL) cpl_table_delete(wl_tab[i]) ;
    }

    /* Return */
    if (cpl_error_get_code()) return -1 ;
    else return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Save the products
  @param    ilist       the wavelength image
  @param    tab         the wavelength tables
  @param    parlist     the parameters list
  @param    set         the frames set
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_util_wlinit_save(
        const cpl_imagelist     *   ilist,
        const cpl_table         **  tab,
        const cpl_parameterlist *   parlist,
        cpl_frameset            *   set)
{
    cpl_propertylist    **  qclists ;
    const cpl_frame     *   ref_frame ;
    cpl_propertylist    *   inputlist ;
    const char          *   recipe_name = "crires_util_wlinit" ;
    int                     i ;

    /* Get the reference frame */
    ref_frame = irplib_frameset_get_first_from_group(set, CPL_FRAME_GROUP_RAW) ;

    /* Create the QC lists */
    qclists = cpl_malloc(CRIRES_NB_DETECTORS * sizeof(cpl_propertylist*)) ;
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        qclists[i] = cpl_propertylist_new() ;
        cpl_propertylist_append_double(qclists[i], "ESO QC CENTWL",
                crires_util_wlinit_config.qc_wlcent[i]) ;
        cpl_propertylist_append_double(qclists[i], "ESO QC DISPWL",
                crires_util_wlinit_config.qc_wldisp[i]) ;

        /* Propagate some keywords from input raw frame extensions */
        inputlist = cpl_propertylist_load_regexp(
                cpl_frame_get_filename(ref_frame), i+1,
                CRIRES_HEADER_EXT_FORWARD, 0) ;
        cpl_propertylist_copy_property_regexp(qclists[i], inputlist, 
                CRIRES_HEADER_EXT_FORWARD, 0) ;
        cpl_propertylist_delete(inputlist) ;
    }

    /* Write the image */
    crires_image_save(set,
            parlist,
            set,
            ilist,
            recipe_name,
            CRIRES_WL_MAP_IMA,
            CRIRES_PROTYPE_WL_MAP,
            crires_util_wlinit_config.period,
            NULL,
            (const cpl_propertylist **)qclists,
            PACKAGE "/" PACKAGE_VERSION,
            "crires_util_wlinit_ima.fits") ;

    /* Write the table */
    crires_table_save(set,
            parlist,
            set,
            tab,
            recipe_name,
            CRIRES_CALPRO_WAVE,
            CRIRES_PROTYPE_WL_POLY,
            NULL,
            (const cpl_propertylist **)qclists,
            PACKAGE "/" PACKAGE_VERSION,
            "crires_util_wlinit_tab.fits") ;

    /* Free and return */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        cpl_propertylist_delete(qclists[i]) ;
    }
    cpl_free(qclists) ;
    return  0;
}

