/* $Id: crires_win_dark.c,v 1.4 2012-09-19 14:52:00 yjung Exp $
 *
 * This file is part of the CRIRES Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: yjung $
 * $Date: 2012-09-19 14:52:00 $
 * $Revision: 1.4 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include "crires_recipe.h"
#include "irplib_utils.h"

/*-----------------------------------------------------------------------------
                                Define
 -----------------------------------------------------------------------------*/

#define RECIPE_STRING "crires_win_dark"

/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static double crires_win_dark_ron(const cpl_image *, const cpl_image *, int) ;
static int crires_win_dark_save(const cpl_imagelist *, 
        const cpl_parameterlist *, cpl_frameset *) ;

static char crires_win_dark_description[] =
"crires_win_dark -- Dark recipe in Windowing Mode\n"
"The files listed in the Set Of Frames (sof-file) must be tagged:\n"
"raw-file.fits "CRIRES_WIN_DARK_RAW".\n" ;

CRIRES_RECIPE_DEFINE(crires_win_dark,
        CRIRES_PARAM_RON_SAMPLES    |
        CRIRES_PARAM_RON_SZ,
        "Dark recipe in Windowing Mode",
        crires_win_dark_description) ;

/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static struct {
    /* Inputs */
    int                 hsize ;
    int                 nsamples ;
    /* Outputs */
    double              dark_med[CRIRES_NB_DETECTORS] ;
    double              dark_stdev[CRIRES_NB_DETECTORS] ;
    double              ron1[CRIRES_NB_DETECTORS] ;
    double              ron2[CRIRES_NB_DETECTORS] ;
} crires_win_dark_config ;

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    parlist     the parameters list
  @param    frameset   the frames list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_win_dark(
        cpl_frameset            *   frameset,
        const cpl_parameterlist *   parlist)
{
    cpl_frameset        *   rawframes ;
    cpl_frame           *   ref_frame ;
    cpl_propertylist    *   plist ;
    double                  dit ;
    int                     ndit ;
    cpl_imagelist       *   darks ;
    cpl_imagelist       *   darks_chip ;
    cpl_image           *   dark[CRIRES_NB_DETECTORS] ;
    cpl_vector          *   medians ;
    double                  med ;
    int                     i, j ;

    /* Initialise */
    rawframes = NULL ;
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        crires_win_dark_config.ron1[i] = -1.0 ;
        crires_win_dark_config.ron2[i] = -1.0 ;
        crires_win_dark_config.dark_med[i] = -1.0 ;
        crires_win_dark_config.dark_stdev[i] = -1.0 ;
        dark[i] = NULL ;
    }

    /* Retrieve input parameters */
    crires_win_dark_config.hsize = crires_parameterlist_get_int(parlist,
            RECIPE_STRING, CRIRES_PARAM_RON_SZ) ;
    crires_win_dark_config.nsamples = crires_parameterlist_get_int(parlist, 
            RECIPE_STRING, CRIRES_PARAM_RON_SAMPLES) ;
 
    /* Identify the RAW and CALIB frames in the input frameset */
    if (crires_dfs_set_groups(frameset, "crires_win_dark")) {
        cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
        return -1 ;
    }

    /* Retrieve raw frames */
    if ((rawframes = crires_extract_frameset(frameset, 
                    CRIRES_WIN_DARK_RAW)) == NULL) {
        cpl_msg_error(__func__, "No raw frame in input") ;
        return -1 ;
    }

    /* At least 3 frames */
    if (cpl_frameset_get_size(rawframes) < 3) {
        cpl_msg_error(__func__, "Not enough input frames");
        cpl_frameset_delete(rawframes) ;
        return -1 ;
    }

    /* Get DIT / NDIT from the header */
    ref_frame = cpl_frameset_get_position(rawframes, 0) ;
    if ((plist=cpl_propertylist_load(cpl_frame_get_filename(ref_frame),
                    0)) == NULL) {
        cpl_msg_error(__func__, "Cannot get header from frame");
        cpl_msg_indent_less() ;
        cpl_frameset_delete(rawframes) ;
        return -1 ;
    }
    dit = crires_pfits_get_dit(plist) ;
    ndit = crires_pfits_get_ndit(plist) ;
    cpl_propertylist_delete(plist) ;
    if (cpl_error_get_code() != CPL_ERROR_NONE) {
        cpl_msg_error(__func__, "Cannot get the DIT/NDIT from the header") ;
        cpl_msg_indent_less() ;
        cpl_frameset_delete(rawframes) ;
        return -1 ;
    }
    cpl_msg_info(__func__, "DIT value: %g sec.", dit) ;
    cpl_msg_info(__func__, "NDIT value: %d", ndit) ;

    /* Loop on the chips */
    cpl_msg_info(__func__, "Dark computation") ;
    cpl_msg_indent_more() ;
    for (i=1 ; i<CRIRES_NB_DETECTORS-1 ; i++) {
        
        /* Load the chips */
        cpl_msg_info(__func__, "Load chip number %d", i+1) ;
        if ((darks_chip = crires_load_frameset(rawframes, 
                        CRIRES_ILLUM_FULL_DETECTOR, i+1, 
                        CPL_TYPE_FLOAT)) == NULL) {
            cpl_msg_error(__func__, "Cannot load chip number %d", i+1) ;
            cpl_msg_indent_less() ;
            cpl_frameset_delete(rawframes) ;
            for (j=1 ; j<i ; j++) 
                cpl_image_delete(dark[j]) ;
            return -1 ;
        }
        /* Compute the current dark */
        cpl_msg_info(__func__, "Collapse images for chip number %d", i+1) ;
        if ((dark[i] = cpl_imagelist_collapse_create(darks_chip)) == NULL) {
            cpl_msg_error(__func__, "Cannot average for chip number %d", i+1) ;
            cpl_msg_indent_less() ;
            cpl_frameset_delete(rawframes) ;
            for (j=1 ; j<i ; j++) 
                cpl_image_delete(dark[j]) ;
            cpl_imagelist_delete(darks_chip) ;
            return -1 ;
        }
        
        /* Compute the dark_med and stdev */
        medians = cpl_vector_new(cpl_imagelist_get_size(darks_chip));
        for (j=0 ; j<cpl_imagelist_get_size(darks_chip) ; j++) {
            med = cpl_image_get_median(cpl_imagelist_get(darks_chip, j)) ;
            cpl_vector_set(medians, j, med) ;
        }
        crires_win_dark_config.dark_med[i] = cpl_vector_get_mean(medians) ;
        crires_win_dark_config.dark_stdev[i] = cpl_vector_get_stdev(medians) ;
        cpl_vector_delete(medians) ;
        
        /* Compute the RONs */
        crires_win_dark_config.ron1[i] = crires_win_dark_ron(
                cpl_imagelist_get(darks_chip, 0),
                cpl_imagelist_get(darks_chip, 1), 
                ndit) ;
        crires_win_dark_config.ron2[i] = crires_win_dark_ron(
                cpl_imagelist_get(darks_chip, 1),
                cpl_imagelist_get(darks_chip, 2), 
                ndit) ;
        cpl_imagelist_delete(darks_chip) ;
    }
    cpl_frameset_delete(rawframes) ;
    cpl_msg_indent_less() ;

    /* Reconstruct chips (windowing mode) using detector 2 */
    if (dark[1] != NULL) {
        dark[0] = cpl_image_duplicate(dark[1]) ;
        cpl_image_multiply_scalar(dark[0], 0.0) ;
        dark[CRIRES_NB_DETECTORS-1] = cpl_image_duplicate(dark[0]) ;
    }

    /* Reconstruct the darks imagelist */
    darks = cpl_imagelist_new() ;
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) 
        cpl_imagelist_set(darks, dark[i], i) ;

    /* Divide by DIT */
    cpl_msg_info(__func__, "Division by DIT") ;
    cpl_imagelist_divide_scalar(darks, dit) ;
    
    /* Save the product */
    cpl_msg_info(__func__, "Save the product") ;
    cpl_msg_indent_more() ;
    if (crires_win_dark_save(darks, parlist, frameset)) {
        cpl_msg_error(__func__, "Cannot save the product") ;
        cpl_imagelist_delete(darks) ;
        cpl_msg_indent_less() ;
        return -1 ;
    }
    cpl_imagelist_delete(darks) ;
    cpl_msg_indent_less() ;

    /* Return */
    if (cpl_error_get_code()) return -1 ;
    else return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    The RON computation
  @param    ima1        the first input image
  @param    ima2        the second input image
  @param    ndit        the NDIT
  @return   the RON or -1 in error case
 */
/*----------------------------------------------------------------------------*/
static double crires_win_dark_ron(
        const cpl_image     *   ima1, 
        const cpl_image     *   ima2, 
        int                     ndit) 
{
    cpl_image       *   ima ;
    double              norm ;
    double              ron ;

    /* Test entries */
    if (ima1 == NULL)   return -1.0 ;
    if (ima2 == NULL)   return -1.0 ;
    if (ndit < 1)       return -1.0 ;

    /* Compute norm */
    norm = 0.5 * ndit ;
    norm = sqrt(norm) ;

    /* Subtraction */
    if ((ima = cpl_image_subtract_create(ima2, ima1)) == NULL) return -1.0 ;
   
    /* RON measurement */
    cpl_flux_get_noise_window(ima, NULL, crires_win_dark_config.hsize,
            crires_win_dark_config.nsamples, &ron, NULL) ;
    cpl_image_delete(ima) ;
    return norm*ron ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Save the products
  @param    dark        the dark images
  @param    parlist     the parameters list
  @param    set         the frames set
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_win_dark_save(
        const cpl_imagelist     *   dark,
        const cpl_parameterlist *   parlist,
        cpl_frameset            *   set)
{
    cpl_propertylist    **  qclists ;
    const cpl_frame     *   ref_frame ;
    cpl_propertylist    *   inputlist ;
    const char          *   recipe_name = "crires_win_dark" ;
    int                     i ;

    /* Get the reference frame */
    ref_frame = irplib_frameset_get_first_from_group(set, CPL_FRAME_GROUP_RAW) ;

    /* Create the QC lists */
    qclists = cpl_malloc(CRIRES_NB_DETECTORS * sizeof(cpl_propertylist*)) ;
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        qclists[i] = cpl_propertylist_new() ;
        cpl_propertylist_append_double(qclists[i], "ESO QC RON1",
                crires_win_dark_config.ron1[i]) ;
        cpl_propertylist_append_double(qclists[i], "ESO QC RON2",
                crires_win_dark_config.ron2[i]) ;
        cpl_propertylist_append_double(qclists[i], "ESO QC DARKMED",
                crires_win_dark_config.dark_med[i]) ;
        cpl_propertylist_append_double(qclists[i], "ESO QC DARKSTDEV",
                crires_win_dark_config.dark_stdev[i]) ;

        /* Propagate some keywords from input raw frame extensions */
        inputlist = cpl_propertylist_load_regexp(
                cpl_frame_get_filename(ref_frame), i+1,
                CRIRES_HEADER_EXT_FORWARD, 0) ;
        cpl_propertylist_copy_property_regexp(qclists[i], inputlist,
                CRIRES_HEADER_EXT_FORWARD, 0) ;
        cpl_propertylist_delete(inputlist) ;
    }

    /* Write the combined image */
    crires_image_save(set,
            parlist,
            set, 
            dark, 
            recipe_name,
            CRIRES_CALPRO_DARK_WIN, 
            CRIRES_PROTYPE_DARK,
            CRIRES_ILLUM_FULL_DETECTOR,
            NULL,
            (const cpl_propertylist**)qclists,
            PACKAGE "/" PACKAGE_VERSION,
            "crires_win_dark.fits") ;

    /* Remove the keywords for the FITS extensions */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        cpl_propertylist_erase_regexp(qclists[i], CRIRES_HEADER_EXT_FORWARD, 0);
    }

    /* Free and return */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        cpl_propertylist_delete(qclists[i]) ;
    }
    cpl_free(qclists) ;
    return  0;
}

