/* $Id: crires_win_flat.c,v 1.7 2012-09-19 14:52:00 yjung Exp $
 *
 * This file is part of the CRIRES Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: yjung $
 * $Date: 2012-09-19 14:52:00 $
 * $Revision: 1.7 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include <locale.h>
#include "crires_recipe.h"
#include "irplib_utils.h"

#include "crires_combine.h"

/*-----------------------------------------------------------------------------
                                Define
 -----------------------------------------------------------------------------*/

#define RECIPE_STRING "crires_win_flat"

/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static cpl_imagelist * crires_win_flat_reduce(cpl_frameset *, const char *, 
        const char *) ;
static cpl_imagelist * crires_win_flat_bpm(cpl_imagelist *, double, 
        double, double) ;
static int crires_win_flat_save(const cpl_imagelist *, const cpl_imagelist *, 
        int, cpl_frameset *, const cpl_parameterlist *, cpl_frameset *) ;
static int crires_win_flat_compare(const cpl_frame *, const cpl_frame *) ;

static char crires_win_flat_description[] =
"crires_win_flat -- Flat-field recipe in Windowing mode\n"
"The files listed in the Set Of Frames (sof-file) must be tagged:\n"
"raw-file.fits "CRIRES_WIN_FLAT_RAW" or\n"
"dark-file.fits "CRIRES_CALPRO_DARK" or\n"
"detlin-file.fits "CRIRES_CALPRO_COEFFS_CUBE".\n" ;

CRIRES_RECIPE_DEFINE(crires_win_flat,
        CRIRES_PARAM_THRESHOLDS     |
        CRIRES_PARAM_BPM_RATE       |
        CRIRES_PARAM_REPLACE        |
        CRIRES_PARAM_KAPPA_SIGCLIP  |
        CRIRES_PARAM_COLLAPSE_METH,
        "Flatfield recipe in Windowing mode",
        crires_win_flat_description) ;

/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static struct {
    /* Input */
    double                  bpm_low ;
    double                  bpm_high ;
    double                  bpm_lines_ratio ;
    int                     replace_flag ;
    double                  kappa_sigclip ;
    crires_collapse_method  coll_meth ;
    /* Output */
    int                     bpm_nb[CRIRES_NB_DETECTORS] ;
    double                  flat_mean[CRIRES_NB_DETECTORS] ;
    double                  flat_stdev[CRIRES_NB_DETECTORS] ;
    double                  flat_flux[CRIRES_NB_DETECTORS] ;
    double                  flat_master_rms[CRIRES_NB_DETECTORS] ;
} crires_win_flat_config ;

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    parlist     the parameters list
  @param    frameset   the frames list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_win_flat(
        cpl_frameset            *   frameset,
        const cpl_parameterlist *   parlist)
{
    const char      *   sval ;
    cpl_size        *   labels ;
    cpl_size            nlabels ;
    cpl_frameset    *   rawframes ;
    const char      *   dark ;
    const char      *   detlin ;
    cpl_imagelist   *   flat ;
    cpl_frameset    *   flat_one ;
    cpl_imagelist   *   bpm ;
    int                 i ;

    /* Needed for sscanf() */
    setlocale(LC_NUMERIC, "C");

    /* Initialise */
    rawframes = NULL ;
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        crires_win_flat_config.bpm_nb[i] = -1 ;
        crires_win_flat_config.flat_mean[i] = -1.0 ;
        crires_win_flat_config.flat_stdev[i] = -1.0 ;
        crires_win_flat_config.flat_flux[i] = -1.0 ;
        crires_win_flat_config.flat_master_rms[i] = -1.0 ;
    }
    
    /* Retrieve input parameters */
    sval = crires_parameterlist_get_string(parlist, RECIPE_STRING, 
            CRIRES_PARAM_THRESHOLDS) ;
    if (sscanf(sval, "%lg,%lg",
                    &crires_win_flat_config.bpm_low,
                    &crires_win_flat_config.bpm_high)!=2) {
        return -1 ;
    }
    crires_win_flat_config.replace_flag = crires_parameterlist_get_bool(
            parlist, RECIPE_STRING, CRIRES_PARAM_REPLACE) ;
    crires_win_flat_config.bpm_lines_ratio = crires_parameterlist_get_double( 
            parlist, RECIPE_STRING, CRIRES_PARAM_BPM_RATE) ;
    crires_win_flat_config.kappa_sigclip = crires_parameterlist_get_double( 
            parlist, RECIPE_STRING, CRIRES_PARAM_KAPPA_SIGCLIP) ;
    sval = crires_parameterlist_get_string(parlist, RECIPE_STRING,
            CRIRES_PARAM_COLLAPSE_METH) ;
    if (!strcmp(sval, "avg"))
        crires_win_flat_config.coll_meth = CRIRES_COLLAPSE_AVG ;
    else if (!strcmp(sval, "med"))
        crires_win_flat_config.coll_meth = CRIRES_COLLAPSE_MED ;
    else if (!strcmp(sval, "sig"))
        crires_win_flat_config.coll_meth = CRIRES_COLLAPSE_SIG ;
    else {
        cpl_msg_error(__func__, "Invalid collapse method specified");
        return -1;
    }
 
    /* Identify the RAW and CALIB frames in the input frameset */
    if (crires_dfs_set_groups(frameset, "crires_win_flat")) {
        cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
        return -1 ;
    }

    /* Retrieve calibration data */
    dark        = crires_extract_filename(frameset, CRIRES_CALPRO_DARK_WIN) ;
    detlin      = crires_extract_filename(frameset, CRIRES_CALPRO_COEFFS_CUBE) ;

    /* Retrieve raw frames */
    if ((rawframes = crires_extract_frameset(frameset, 
                    CRIRES_WIN_FLAT_RAW)) == NULL) {
        cpl_msg_error(__func__, "No raw frame in input") ;
        return -1 ;
    }

    /* Labelise all input frames */
    if ((labels = cpl_frameset_labelise(rawframes, crires_win_flat_compare,
                &nlabels)) == NULL) {
        cpl_msg_error(__func__, "Cannot labelise input frames") ;
        cpl_frameset_delete(rawframes) ;
        return -1 ;
    }

    /* Extract settings and reduce each of them */
    for (i=0 ; i<(int)nlabels ; i++) {
        /* Reduce data set nb i */
        cpl_msg_info(__func__, "Reduce data set %d / %"CPL_SIZE_FORMAT, 
                i+1, nlabels);
        cpl_msg_indent_more() ;
        flat_one = cpl_frameset_extract(rawframes, labels, (cpl_size)i) ;
        flat = crires_win_flat_reduce(flat_one, dark, detlin) ;
        cpl_msg_indent_less() ;

        /* Save the products */
        cpl_msg_info(__func__, "Save the products") ;
        cpl_msg_indent_more() ;
        if (flat == NULL) {
            cpl_msg_warning(__func__, "Cannot reduce set nb %d", i+1) ;
        } else {
            if ((bpm = crires_win_flat_bpm(flat,
                            crires_win_flat_config.bpm_low,
                            crires_win_flat_config.bpm_high,
                            crires_win_flat_config.bpm_lines_ratio)) == NULL) {
                cpl_msg_warning(__func__, "Cannot create bad pixels map") ;
            }
            crires_win_flat_save(flat, bpm, i+1, flat_one, parlist, frameset) ;
            cpl_imagelist_delete(flat) ;
            cpl_imagelist_delete(bpm) ;
        }
        cpl_msg_indent_less() ;
        cpl_frameset_delete(flat_one) ;
    }
    cpl_frameset_delete(rawframes) ;
    cpl_free(labels) ;

    /* Return */
    if (cpl_error_get_code()) return -1 ;
    else return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    The recipe data reduction part is implemented here 
  @param    flatframes  the frames list with flat frames
  @param    dark        the dark frame or NULL
  @param    detlin      the detlin frame or NULL
  @return   the flat images
 */
/*----------------------------------------------------------------------------*/
static cpl_imagelist * crires_win_flat_reduce(
        cpl_frameset    *   flatframes,
        const char      *   dark,
        const char      *   detlin)
{
    cpl_propertylist    *   plist ;
    cpl_frame           *   ref_frame ;
    const char          *   fname ;
    int                     nframes ;
    double                  dit_frame, dit_dark ;
    cpl_imagelist       *   in ;
    cpl_imagelist       *   out ;
    cpl_vector          *   medians ;
    double                  median ;
    cpl_image           *   ima[CRIRES_NB_DETECTORS] ;
    int                     offset_y, stripe_y, center_y ;
    int                     i, j, k ;

    /* Test entries */
    if (flatframes == NULL) return NULL ;

    /* Initialize */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        ima[i] = NULL ;
    }
    nframes = cpl_frameset_get_size(flatframes) ;

    /* Get the DIT from the RAW frame */
    ref_frame = cpl_frameset_get_position(flatframes, 0) ;
    fname = cpl_frame_get_filename(ref_frame) ;
    if ((plist=cpl_propertylist_load(fname, 0)) == NULL) {
        cpl_msg_error(__func__, "Getting header from RAW file");
        cpl_msg_indent_less() ;
        return NULL ;
    }
    dit_frame = crires_pfits_get_dit(plist) ;
    offset_y = crires_pfits_get_stripe_offsety(plist) ;
    center_y = crires_pfits_get_stripe_centery(plist) ;
    cpl_propertylist_delete(plist) ;
    if (cpl_error_get_code()) {
        cpl_msg_error(__func__, 
                "Cannot get the DIT or STRIPE keywords from RAW file") ;
        cpl_msg_indent_less() ;
        return NULL ;
    }
    cpl_msg_info(__func__, "DIT value: %g sec.", dit_frame) ;
    
    /* Verify the DIT of the dark */
    if (dark != NULL) {
        cpl_msg_info(__func__, "Verify the dark DIT") ;
        cpl_msg_indent_more() ;
        if ((plist=cpl_propertylist_load(dark, 0)) == NULL) {
            cpl_msg_error(__func__, "Getting header from DARK");
            cpl_msg_indent_less() ;
            return NULL ;
        }
        dit_dark = crires_pfits_get_dit(plist) ;
        cpl_propertylist_delete(plist) ;
        if (cpl_error_get_code()) {
            cpl_msg_error(__func__, "Cannot get the DIT from DARK") ;
            cpl_msg_indent_less() ;
            return NULL ;
        }
        if (fabs(dit_dark-dit_frame) > 1e-5) {
            cpl_msg_error(__func__, "Mismatch RAW DIT (%g) / DARK DIT (%g)",
                    dit_frame, dit_dark) ;
            cpl_msg_indent_less() ;
            return NULL ;
        }
        cpl_msg_indent_less() ;

        /* Verify the STRIPE keys conformity */
        if (crire_stripe_keys_mismatch(fname, dark)) {
            cpl_msg_error(__func__, 
                    "Mismatch of STRIPE keys with the dark frame") ;
            return NULL ;
        }
    }

    /* Loop on the detectors */
    for (i=1 ; i<CRIRES_NB_DETECTORS-1 ; i++) {
        cpl_msg_info(__func__, "Compute the MASTER FLAT for chip nb %d", i+1) ;
        cpl_msg_indent_more() ;

        /* Load the data */
        in = crires_load_frameset(flatframes, CRIRES_ILLUM_FULL_DETECTOR, 
                i+1, CPL_TYPE_FLOAT) ;
        stripe_y = (int)(cpl_image_get_size_y(cpl_imagelist_get(in, 0))/2.0) ;

        /* Correct for dark */
        if (crires_calib_chip_list(in, CRIRES_ILLUM_FULL_DETECTOR, i+1, 
                    NULL, dark, NULL, detlin, dit_frame, center_y, offset_y, 
                    stripe_y)) {
            cpl_msg_error(__func__, "Cannot apply the calibrations") ;
            cpl_imagelist_delete(in) ;
            for (j=1 ; j<i ; j++) 
                cpl_image_delete(ima[j]);
            return NULL ;
        }

        /* Create medians vector */
        medians = cpl_vector_new(nframes) ;

        /* Loop on all the frames */
        cpl_msg_info(__func__, "Normalise with the median") ;
        for (j=0 ; j<nframes ; j++) {
            median = cpl_image_get_median(cpl_imagelist_get(in, j)) ;
            if (cpl_error_get_code()) {
                cpl_msg_error(__func__, "Cannot compute the median") ;
                cpl_imagelist_delete(in) ;
                cpl_vector_delete(medians) ;
                for (k=1 ; k<i ; k++) 
                    cpl_image_delete(ima[k]);
                return NULL ;
            }
            cpl_vector_set(medians, j, median) ;
            if (fabs(median) > 1e-3) 
                cpl_image_divide_scalar(cpl_imagelist_get(in, j), median) ;
        }

        /* Fill QCs */
        crires_win_flat_config.flat_mean[i] = cpl_vector_get_mean(medians) ;
        if (cpl_vector_get_size(medians) > 1) 
            crires_win_flat_config.flat_stdev[i]=cpl_vector_get_stdev(medians);
        else
            crires_win_flat_config.flat_stdev[i] = -1.0 ;
        crires_win_flat_config.flat_flux[i] =
            crires_win_flat_config.flat_mean[i] / dit_frame ;
       
        /* Collapse the frames */
        if ((ima[i] = crires_combine_collapse_imagelist(in, medians, 
                        crires_win_flat_config.kappa_sigclip,
                        i+1,
                        crires_win_flat_config.coll_meth)) == NULL) {
            cpl_msg_error(__func__, "Cannot average the flats") ;
            cpl_imagelist_delete(in) ;
            cpl_vector_delete(medians) ;
            for (j=1 ; j<i ; j++) 
                cpl_image_delete(ima[j]);
            return NULL ;
        }
        cpl_vector_delete(medians) ;
        cpl_imagelist_delete(in) ;

        /* Add QC */
        crires_win_flat_config.flat_master_rms[i] =
            cpl_image_get_stdev(ima[i]) ;
        
        cpl_msg_indent_less() ;
    }

    /* Reconstruct chips (windowing mode) using detector 2 */
    if (ima[1] != NULL) {
        ima[0] = cpl_image_duplicate(ima[1]) ;
        cpl_image_multiply_scalar(ima[0], 0.0) ;
        ima[CRIRES_NB_DETECTORS-1] = cpl_image_duplicate(ima[0]) ;
    }
    
    /* Create the image list */
    out = cpl_imagelist_new() ;
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) 
        cpl_imagelist_set(out, ima[i], i) ;

    return out ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    The BPM computation
  @param    flat    the master flat images
  @param    low     the low threshold
  @param    high    the high threshold
  @param    bad_per_line_limit  The maximum ration of bad pixels per line
  @return   the flat images
 */
/*----------------------------------------------------------------------------*/
static cpl_imagelist * crires_win_flat_bpm(
        cpl_imagelist       *   flat,
        double                  low,
        double                  high,
        double                  bad_per_line_limit)
{
    cpl_imagelist   *   bpm ;
    int                 nima ;
    cpl_image       *   bpm_cur ;
    cpl_mask        *   mask_cur ;
    cpl_binary      *   pmask_cur ;
    int                 nx, ny, cur_bp_nb ;
    int                 i, j, k ;

    /* Test entries */
    if (flat == NULL) return NULL ;

    /* Initialise */
    nima = cpl_imagelist_get_size(flat) ;

    /* Create the output image list */
    bpm = cpl_imagelist_new() ;

    /* Loop on the images */
    for (i=0 ; i<nima ; i++) {
        /* Threshold to get the BPMs */
        if ((mask_cur = cpl_mask_threshold_image_create(
                        cpl_imagelist_get(flat, i), low, high)) == NULL) {
            cpl_msg_error(__func__, "Cannot create bad pixels map") ;
            cpl_imagelist_delete(bpm) ;
            return NULL ;
        }
        cpl_mask_not(mask_cur) ;

        /*
        Post processing : Big zones of bad pixels are not considered as
        bad pixels. Each line containing more than
        100*bad_per_line_limit percent bad pixels is reset to contain
        anly good pixels.
        */
        nx = cpl_mask_get_size_x(mask_cur) ;
        ny = cpl_mask_get_size_y(mask_cur) ;
        pmask_cur = cpl_mask_get_data(mask_cur) ;
        for (j=0 ; j<ny ; j++) {
            cur_bp_nb = cpl_mask_count_window(mask_cur, 1, j+1, nx, j+1) ;
            /* Check if the line has too many bad pixels */
            if (cur_bp_nb > bad_per_line_limit * nx) {
                /* Reset the bad pixels on the current line */
                for (k=0 ; k<nx ; k++) {
                    pmask_cur[k+j*nx] = CPL_BINARY_0 ;
                }
            }
        }

        /* Convert mask to image */
        bpm_cur = cpl_image_new_from_mask(mask_cur) ;
        crires_win_flat_config.bpm_nb[i] = cpl_mask_count(mask_cur) ;
        cpl_mask_delete(mask_cur) ;
        cpl_imagelist_set(bpm, bpm_cur, i) ;

        /* Set the flat to 1 outside of the bounds */
        if (crires_win_flat_config.replace_flag) {
            cpl_image_threshold(cpl_imagelist_get(flat, i),
                    low, high, 1.0, 1.0) ;
        }
    }
    return bpm ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Save the products
  @param    flat        the flat images
  @param    set_nb      the setting number
  @param    set         the frames set
  @param    parlist     the parameters list
  @param    set_tot     the total frames set
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_win_flat_save(
        const cpl_imagelist     *   flat,
        const cpl_imagelist     *   bpm,
        int                         set_nb,
        cpl_frameset            *   set,
        const cpl_parameterlist *   parlist,
        cpl_frameset            *   set_tot)
{
    cpl_propertylist    **  qclists ;
    const cpl_frame     *   ref_frame ;
    char                *   filename ;
    cpl_propertylist    *   inputlist ;
    const char          *   recipe_name = "crires_win_flat" ;
    int                     i ;

    /* Get the reference frame */
    ref_frame = irplib_frameset_get_first_from_group(set, CPL_FRAME_GROUP_RAW) ;

    /* Create the QC lists */
    qclists = cpl_malloc(CRIRES_NB_DETECTORS * sizeof(cpl_propertylist*)) ;
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        qclists[i] = cpl_propertylist_new() ;
        cpl_propertylist_append_int(qclists[i], "ESO QC NBBAD",
                crires_win_flat_config.bpm_nb[i]) ;
        cpl_propertylist_append_double(qclists[i], "ESO QC FLAT MEAN",
                crires_win_flat_config.flat_mean[i]) ;
        cpl_propertylist_append_double(qclists[i], "ESO QC FLAT STDEV",
                crires_win_flat_config.flat_stdev[i]) ;
        cpl_propertylist_append_double(qclists[i], "ESO QC FLAT FLUX",
                crires_win_flat_config.flat_flux[i]) ;
        cpl_propertylist_append_double(qclists[i], "ESO QC FLAT MASTER RMS",
                crires_win_flat_config.flat_master_rms[i]) ;

        /* Propagate some keywords from input raw frame extensions */
        inputlist = cpl_propertylist_load_regexp(
                cpl_frame_get_filename(ref_frame), i+1,
                CRIRES_HEADER_EXT_FORWARD, 0) ;
        cpl_propertylist_copy_property_regexp(qclists[i], inputlist, 
                CRIRES_HEADER_EXT_FORWARD, 0) ;
        cpl_propertylist_delete(inputlist) ;
    }

    /* Write the flat image */
    filename = cpl_sprintf("%s_set%02d.fits", recipe_name, set_nb) ;
    crires_image_save(set_tot,
            parlist,
            set, 
            flat, 
            recipe_name,
            CRIRES_CALPRO_FLAT_WIN, 
            CRIRES_PROTYPE_FLAT,
            CRIRES_ILLUM_FULL_DETECTOR,
            NULL,
            (const cpl_propertylist**)qclists, 
            PACKAGE "/" PACKAGE_VERSION,
            filename) ;
    cpl_free(filename) ;

    /* Write the BPM */
    if (bpm != NULL) {
        filename = cpl_sprintf("%s_set%02d_bpm.fits", recipe_name, set_nb) ;
        crires_image_save(set_tot,
                parlist,
                set, 
                bpm, 
                recipe_name,
                CRIRES_CALPRO_BPM_WIN, 
                CRIRES_PROTYPE_BPM,
                CRIRES_ILLUM_FULL_DETECTOR,
                NULL,
                (const cpl_propertylist**)qclists, 
                PACKAGE "/" PACKAGE_VERSION,
                filename) ;
        cpl_free(filename) ;
    }

    /* Free and return */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        cpl_propertylist_delete(qclists[i]) ;
    }
    cpl_free(qclists) ;
    return  0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Comparison function to identify different settings
  @param    frame1  first frame 
  @param    frame2  second frame 
  @return   0 if frame1!=frame2, 1 if frame1==frame2, -1 in error case
 */
/*----------------------------------------------------------------------------*/
static int crires_win_flat_compare(
        const cpl_frame   *   frame1,
        const cpl_frame   *   frame2)
{
    int                     comparison ;
    cpl_propertylist    *   plist1 ;
    cpl_propertylist    *   plist2 ;
    double                  dval1, dval2 ;

    /* Test entries */
    if (frame1==NULL || frame2==NULL) return -1 ;

    /* Get property lists */
    if ((plist1=cpl_propertylist_load(cpl_frame_get_filename(frame1),0))==NULL){
        cpl_msg_error(__func__, "getting header from reference frame");
        return -1 ;
    }
    if ((plist2=cpl_propertylist_load(cpl_frame_get_filename(frame2),0))==NULL){
        cpl_msg_error(__func__, "getting header from reference frame");
        cpl_propertylist_delete(plist1) ;
        return -1 ;
    }

    /* Test status */
    if (cpl_error_get_code()) {
        cpl_propertylist_delete(plist1) ;
        cpl_propertylist_delete(plist2) ;
        return -1 ;
    }

    comparison = 1 ;

    /* Compare the DIT used */
    dval1 = crires_pfits_get_dit(plist1) ;
    dval2 = crires_pfits_get_dit(plist2) ;
    if (cpl_error_get_code()) {
        cpl_msg_error(__func__, "Cannot get the DIT");
        cpl_propertylist_delete(plist1) ;
        cpl_propertylist_delete(plist2) ;
        return -1 ;
    }
    if (fabs(dval1-dval2) > 1e-3) comparison = 0 ;

    /* Compare the WLEN REF used */
    dval1 = crires_pfits_get_refwlen(plist1) ;
    dval2 = crires_pfits_get_refwlen(plist2) ;
    if (cpl_error_get_code()) {
        cpl_msg_error(__func__, "Cannot get the reference wavelength");
        cpl_propertylist_delete(plist1) ;
        cpl_propertylist_delete(plist2) ;
        return -1 ;
    }
    if (fabs(dval1-dval2) > 1e-3) comparison = 0 ;

    /* Compare the SHUT1 POS used */
    dval1 = crires_pfits_get_bafflepos(plist1) ;
    dval2 = crires_pfits_get_bafflepos(plist2) ;
    if (cpl_error_get_code()) {
        cpl_msg_error(__func__, "Cannot get the baffle position");
        cpl_propertylist_delete(plist1) ;
        cpl_propertylist_delete(plist2) ;
        return -1 ;
    }
    if (fabs(dval1-dval2) > 1e-3) comparison = 0 ;

    cpl_propertylist_delete(plist1) ;
    cpl_propertylist_delete(plist2) ;
    return comparison ;
}

